Package: TFutils
Title: TFutils
Description: This package helps users to work with TF metadata from various sources.
	Significant catalogs of TFs and classifications thereof are made available.
	Tools for working with motif scans are also provided.
Version: 1.32.0
Authors@R: c(
 person("Vincent", "Carey", role = c("aut", "cre"), email = "stvjc@channing.harvard.edu"),
 person("Shweta", "Gopaulakrishnan", role = c("aut"), email = "reshg@channing.harvard.edu"))
Depends: R (>= 4.1.0)
Imports: methods, dplyr, magrittr, miniUI, shiny, Rsamtools, GSEABase,
        rjson, BiocFileCache, DT, httr, readxl, AnnotationDbi,
        org.Hs.eg.db, utils, GenomicFiles, SummarizedExperiment
Suggests: knitr, data.table, testthat, AnnotationFilter, Biobase,
        GenomicFeatures, GenomicRanges, Gviz, IRanges, S4Vectors,
        EnsDb.Hsapiens.v75, BiocParallel, BiocStyle, GO.db, Seqinfo,
        UpSetR, ggplot2, png, gwascat, MotifDb, motifStack,
        RColorBrewer, rmarkdown
License: Artistic-2.0
LazyLoad: true
ByteCompile: true
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.2
biocViews: Transcriptomics
Roxygen: list(markdown = TRUE)
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/TFutils
git_branch: RELEASE_3_23
git_last_commit: ac50301
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:10:46 UTC; biocbuild
Author: Vincent Carey [aut, cre],
  Shweta Gopaulakrishnan [aut]
Maintainer: Vincent Carey <stvjc@channing.harvard.edu>

# Generated by roxygen2: do not edit by hand

export(HGNCmap)
export(TFCatalog)
export(TFtargs)
export(URL_s3_tf)
export(anchor_pmids)
export(browse_gotf_main)
export(browse_humantfs_main)
export(browse_lambert_gwaslinks)
export(browse_lambert_main)
export(defaultCircosParms)
export(directHitsInCISBP)
export(fimo_granges)
export(genemodForGviz)
export(genemodelDF)
export(get_rslocs_38)
export(grabTab)
export(importFIMO_local_split)
export(retrieve_gotf_main)
export(retrieve_humantfs_main)
export(retrieve_lambert_main)
export(tffamCirc.plot)
export(tffamCirc.prep)
export(topTraitsOfTargets)
exportClasses(TFCatalog)
exportMethods(importFIMO)
exportMethods(show)
import(AnnotationDbi, except=select)
import(BiocFileCache)
import(SummarizedExperiment)
import(dplyr)
import(magrittr)
import(miniUI)
import(org.Hs.eg.db)
import(readxl)
importClassesFrom(GSEABase,GeneSetCollection)
importClassesFrom(Rsamtools,TabixFile)
importFrom(GSEABase,SymbolIdentifier)
importFrom(GSEABase,geneIdType)
importFrom(GSEABase,geneIds)
importFrom(GSEABase,mapIdentifiers)
importFrom(GenomicFiles,reduceByRange)
importFrom(Rsamtools,scanTabix)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(shiny,dataTableOutput)
importFrom(shiny,observeEvent)
importFrom(shiny,reactive)
importFrom(shiny,renderDataTable)
importFrom(shiny,runGadget)
importFrom(shiny,selectInput)
importFrom(shiny,stopApp)
importFrom(stats,family)
importFrom(stats,na.omit)
importFrom(utils,read.csv)
importFrom(utils,read.csv2)
importFrom(utils,read.delim)

THE SOURCE IS IN vignettes/TFutils, generated by "draft"!!

rmarkdown::draft(file = "TFutils.Rmd", 
                 template = "f1000_article", 
                 package = "BiocWorkflowTools", 
                 edit = FALSE)

#' @importClassesFrom Rsamtools TabixFile
setGeneric("importFIMO", function(src, parms, ...) standardGeneric("importFIMO"))

#' import a FIMO bed-like file
#' @importFrom utils read.delim
#' @rdname importFIMO
#' @aliases importFIMO,TabixFile,GRanges-method importFIMO
#' @param src TabixFile instance
#' @param parms a GRanges instance delimiting the import; multiple GRanges can be used
#' @param \dots passed to GenomicRanges::GRanges
#' @return instance of GRanges
#' @examples
#' if (requireNamespace("Rsamtools")) {
#'  tf = Rsamtools::TabixFile(system.file("M5946_1/chr1.bed.gz", package="TFutils"))
#'  importFIMO(tf, GenomicRanges::GRanges("chr1", IRanges::IRanges(1e6,11e6)))
#'  }
#' @export
setMethod("importFIMO", c("TabixFile", "GRanges"), function(src, parms, ...) {
  jnk = lapply(c("Seqinfo", "Rsamtools", "GenomicRanges", "IRanges",
    "Seqinfo"), reqNS)
  tmp = Rsamtools::scanTabix(src, param=parms) # list with one element per range in parms
  dfs = lapply(tmp, function(x) utils::read.delim(textConnection(x), header=FALSE))
  alldf = do.call(rbind, dfs)
  GenomicRanges::GRanges(alldf$V1, IRanges::IRanges(
      start=alldf$V2, end=alldf$V3), score=alldf$V5, pvalue=alldf$V7, strand=alldf$V6, 
      seqinfo=Seqinfo::seqinfo(parms), ...)
})

#' @rdname importFIMO
#' @aliases importFIMO,character,missing-method
#' @export
setMethod("importFIMO", c("character", "missing"), function(src, parms, ...) {
  if (!requireNamespace("data.table")) stop("install data.table to use this function")
  stopifnot(file.exists(src)) # assume textual bed-like file
  tmp = data.table::fread(src)
  GenomicRanges::GRanges(tmp$V1, IRanges::IRanges(
      start=tmp$V2, end=tmp$V3), score=tmp$V5, pvalue=tmp$V7, strand=tmp$V6, 
      ...)
})

reqNS = function(pname) if (!requireNamespace(pname)) {
  stop(paste("please install", pname, "to use this function"))
}

#' use EnsDb to generate an exon-level model of genes identified by symbol
#' @param sym a character() vector of gene symbols
#' @param resource should be or inherit from EnsDb, 
#' answering exons(), with AnnotationFilter::SymbolFilter 
#' as filter parameter
#' @param columnsKept character vector used as `columns` param in exons()
#' @param \dots passed to exons()
#' @note There are many approaches available to acquiring 'gene models'
#' in Bioconductor; this one emphasizes the use of the exons method
#' for Ensembl annotation.
#' @return data.frame instance with exons in rows
#' @examples
#' if (requireNamespace("EnsDb.Hsapiens.v75")) {
#'  orm = genemodelDF("ORMDL3", EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75)
#'  dim(orm)
#' }
#' head(orm)
#' @export
genemodelDF = function(sym, resource, 
      columnsKept=c("gene_id", "tx_id"), ...) {
  if (is.character(resource)) reqNS(resource)
  reqNS("AnnotationFilter")
  reqNS("GenomicFeatures")
  if (!is(resource, "EnsDb")) warning("resource not of class EnsDb")
  exs = GenomicFeatures::exons(resource, filter=AnnotationFilter::SymbolFilter(sym), 
         columns=columnsKept, ...)
  names(exs) = NULL
  as(exs, "data.frame")
}

#' create a GeneRegionTrack instance for selected symbols
#' @param sym character vector of gene symbols, should be neighboring genes
#' @param id_elem vector of names of columns generated by genemodelDF to be
#' used to label transcripts
#' @param resource should be or inherit from EnsDb, 
#' answering exons(), with AnnotationFilter::SymbolFilter 
#' as filter parameter
#' @note This function helps to display the locations of TF binding sites
#' in the context of complex gene models.  A complication is that we
#' have nice visualization of quantitative affinity predictions for TFs
#' in the vignette, based on ggplot2, but it is not clear how to use
#' that specific code to work with Gviz.
#' @param \dots passed to genemodelDF
#' @return instance of Gviz GeneRegionTrack
#' @examples
#' if (requireNamespace("EnsDb.Hsapiens.v75") &
#'     requireNamespace("Gviz")) {
#'  orm = genemodForGviz("ORMDL3", resource= EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75)
#'  orm
#'  Gviz::plotTracks(orm, showId=TRUE) # change id_elem for shorter id string
#' }
#' @export
genemodForGviz = function(sym="ORMDL3", id_elem=c("symbol", "tx_id"),
        resource = EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75, ...) {
 reqNS("Gviz")
 exdf = genemodelDF(sym=sym, resource=resource, ...)
 cind = which(names(exdf)=="seqnames")
 if (length(cind)==1) names(exdf)[cind] = "chromosome"
 Gviz::GeneRegionTrack(exdf, transcript=exdf$tx_id, 
     symbol = apply(exdf[,id_elem,drop=FALSE],1, paste, collapse=" "), ...)
}

library(TFutils)
library(GSEABase)
library(GenomicRanges)

context("table/range generation")

test_that("grabTab returns expected records", {
  tt = grabTab("VDR", gscoll=TFutils::tftColl,
     orgdb=org.Hs.eg.db::org.Hs.eg.db, 
     gwrngs=TFutils::gwascat_hg19_chr17)
  expect_true(all(dim(tt)==c(28,6)))
})

test_that("tfbs_as_GRanges retrieves scored intervals from local bed files", {
 localtf = system.file("M5946_1/chr1.bed.gz", package="TFutils")
 gr = GenomicRanges::GRanges("chr1", IRanges::IRanges(1e6,11e6))
 oo = importFIMO(Rsamtools::TabixFile(localtf), parms= gr)
 expect_true(length(oo)==10)
})

test_that("genemodelDF generates expected number of records", {
 if (requireNamespace("EnsDb.Hsapiens.v75")) {
  orm = genemodelDF("ORMDL3", EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75)
  expect_true(all(dim(orm)==c(29,9)))
  }
})

context("URL generation")

test_that("URL_s3_tf finds supplied gene symbol", {
 expect_true(URL_s3_tf("VDR") == "http://s3.amazonaws.com/biocfound-tfbs/M5946_1.02sort.bed.gz")
})

test_that("URL_s3_tf finds supplied gene symbol", {
 expect_true(URL_s3_tf("M5946_1") == "http://s3.amazonaws.com/biocfound-tfbs/M5946_1.02sort.bed.gz")
})

#' utility to read FIMO outputs from local resource(cluster), assuming bed text split by chromosome
#' @param tf character(1) file id
#' @param chr character(1) chromosome name
#' @return data.table instance
#' @examples
#' requireNamespace("GenomicRanges")
#' requireNamespace("IRanges")
#  requireNamespace("DT")
#' importFIMO_local_split("M5946_1", "chr1")
#' dim(importFIMO_local_split("M5946_1", "chr17"))
#' @export
importFIMO_local_split = function( tf, chr ) {
  reqNS("IRanges")
  reqNS("GenomicRanges")
  if (!requireNamespace("data.table")) stop("install data.table to use this function")
  stopifnot(length(tf)==1, is(tf, "character"))
  chromosome = chr
  myfile = system.file(paste0(tf,"/",chromosome,".bed"), package="TFutils")
  chrbed = data.table::fread(myfile)
  chrbed
  #chr = plyr::rename(chrbed, c("V1"= "chr","V2"="start","V3"="end","V4"="interval","V5"="score","V6"="strand","V7"="pvalue"))
}

#' Update Oct 18 2021

gotf_url = function() {
   "https://www.biorxiv.org/content/biorxiv/early/2020/12/07/2020.10.28.359232/DC1/embed/media-1.xlsx"
}

#' acquire the content of Table S1.A from Lovering et al., A GO catalogue of human DNA-binding 
#' transcription factors, DOI: https://doi.org/10.1101/2020.10.28.359232
#' @rawNamespace import(AnnotationDbi, except=select)
#' @import BiocFileCache
#' @import readxl
#' @import org.Hs.eg.db
#' @param cache a BiocFileCache instance
#' @return a tbl_df
#' @note This will download the spreadsheet if not found in `cache`.
#' @examples
#' if (interactive()) retrieve_gotf_main()
#' @export
#' 
#' 
retrieve_gotf_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
  uuid_element = "www.biorxiv.org/content/biorxiv/early/2020/12/07/2020.10.28.359232/DC1/embed/media-1.xlsx"
  peek = BiocFileCache::bfcquery(cache, uuid_element)
  if(length(peek$rpath)==0) 
    BiocFileCache::bfcadd(cache, gotf_url())
  peek = try(BiocFileCache::bfcquery(cache, uuid_element))
  if (inherits(peek, "try-error") || length(peek$rpath)==0) stop('could not retrieve gotf')
    ans = suppressMessages(suppressWarnings(readxl::read_xlsx(peek$rpath, skip=0, sheet=2))) 
  ans
}


#' use DT::datatable to browse the GO catalogue of human DNA-binding transcription factors in Table S1.A of Lovering et al.
#' @param cache a BiocFileCache instance
#' @return result of DT::datatable
#' @examples
#' if (interactive()) browse_gotf_main()
#' @export
browse_gotf_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
  tab = as.data.frame(retrieve_gotf_main(cache=cache))
  names(tab) <- gsub(" ", "_", names(tab))
  idx = tab$UniProt_ID != 'pseudogene' # keep only protein coding genes
  tab <- tab[ idx, ]
  ids = AnnotationDbi::mapIds(org.Hs.eg.db, tab$HGNC_approved_gene_symbol, "ENSEMBL", "SYMBOL", multiVals="list") # add Ensembl Ids 
  tab$ENSEMBLID <- ids
  DT::datatable(tab, escape=FALSE)
  
}

#' create a list of GRanges for FIMO hits in a GenomicFiles instance, corresponding to a GRanges-based query
#' @importFrom Rsamtools scanTabix
#' @importFrom GenomicFiles reduceByRange
#' @import SummarizedExperiment
#' @param gf GenomicFiles instance, like fimo16 in TFutils
#' @param query a GRanges specifying ranges to check for TF binding scores
#' @return a list of GRanges, produced by GenomicFiles::reduceByRange 
#' @note Be sure to use `register([BPPARAM])` appropriately.
#' @examples
#' if (interactive()) {   # need internet
#'  # setup -- annotate fimo16 object and create an informative
#'  # query
#'  colnames(fimo16) = fimo16$HGNC
#'  si = Seqinfo::Seqinfo(genome="hg19")["chr17"] # to fix query genome
#'  myg = GRanges("chr17", IRanges(38.07e6,38.09e6), seqinfo=si)
#'  requireNamespace("BiocParallel")
#'  BiocParallel::register(BiocParallel::SerialParam())
#'  f1 = fimo_granges(fimo16[, c("VDR", "POU2F1")], myg)
#'  f1
#' }
#' @export
fimo_granges = function (gf, query) {
    SummarizedExperiment::rowRanges(gf) <- query
    proctext = function(x) {
        con = textConnection(x)
        on.exit(close(con))
        dtf = read.delim(con, h = FALSE, stringsAsFactors = FALSE, 
            sep = "\t")
        colnames(dtf) = c("chr", "start", "end", "rname", "score", 
            "dir", "pval")
        ans = with(dtf, GRanges(seqnames = chr, IRanges(start, 
            end), rname = rname, score = score, dir = dir, pval = pval))
        ans
    }
    nfun = function(ans) lapply(ans, lapply, proctext)
    ans = GenomicFiles::reduceByRange(gf, MAP = function(r, f) scanTabix(f, 
        param = r), REDUCE = nfun)
    if (ncol(gf) == 1) {  # REDUCE IS NOT INVOKED FOR ncol(fg) == 1
        ans = lapply(ans, function(z) proctext(z[[1]]))
        ans = lapply(ans, function(z) {
          Seqinfo::seqinfo(z) = Seqinfo::seqinfo(query)
          z
        })
       }
    else {
        ans = unlist(ans, recursive = FALSE)
        ans = lapply(ans, lapply, function(x) {
            Seqinfo::seqinfo(x) = Seqinfo::seqinfo(query)
            x
        })
    }
    names(ans) = rep(colnames(gf), length(query))
    ans
}

#fimo_granges_OLD = function (gf, query) 
#{
#    rowRanges(gf) = query
#    nfun = function(ans) lapply(ans, lapply, function(x) {
#        con = textConnection(x)
#        on.exit(close(con))
#        dtf = read.delim(con, h = FALSE, stringsAsFactors = FALSE, 
#            sep = "\t")
#        colnames(dtf) = c("chr", "start", "end", "rname", "score", 
#            "dir", "pval")
#        ans = with(dtf, GRanges(seqnames = chr, IRanges(start, 
#            end), rname = rname, score = score, dir = dir, pval = pval))
#        ans
#    })
#    ans = reduceByRange(gf, MAP = function(r, f) scanTabix(f, 
#        param = r), REDUCE = nfun)
#    ans = unlist(ans, recursive=FALSE)
#    names(ans) = rep(colnames(gf), length(query))
#    lapply(ans, lapply, function(x) {Seqinfo::seqinfo(x) = Seqinfo::seqinfo(query); x})
#}

#' gadget to help sort through tags naming TFs
#' @importFrom shiny selectInput dataTableOutput reactive renderDataTable observeEvent
#' @importFrom shiny runGadget stopApp
#' @import miniUI
#' @param gscoll a GSEABase GeneSetCollection
#' @param initTF character(1) initial TF string for app
#' @param gwcat GRanges-like structure with GWAS catalog information
#' @param gadtitle character(1) a title for the gadget panel
#' @note Will use TFutils::gwascat_hg19_chr17 to look for 'MAPPED_GENE' field entries matching targets, also hardcoded to use org.Hs.eg.db to map symbols
#' @return on app conclusion a data.frame is returned
#' @examples
#' if (interactive()) TFtargs()
#' @export
TFtargs = function(gscoll=TFutils::tftColl, initTF="VDR_Q3",
   gwcat = TFutils::gwascat_hg19_chr17,
   gadtitle="Search for a TF; its targets will be checked for mapped status in GWAS catalog") {
  ui <- miniPage(gadgetTitleBar(gadtitle), 
                 miniContentPanel(
                    selectInput("tfsel", "TF:", sapply(gscoll, GSEABase::setName),
                         selected = initTF),
                    dataTableOutput("tab"))
                ) # end page
  server <- function(input, output, session) {
    getTab = reactive({
      grabTab(input$tfsel, TFutils::tftColl, org.Hs.eg.db::org.Hs.eg.db, gwcat)
      })
    output$tab <- renderDataTable({
      getTab()
    })
    observeEvent(input$done, {
      stopApp(getTab())
    })
  }
  runGadget(ui, server)
}

# this code attempts to transform the PMIDs in 
# comments into suitable HTML anchors

eight_digits_regexp = function(x) 
    "[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]"
seven_digits_regexp = function(x) 
    "[0-9][0-9][0-9][0-9][0-9][0-9][0-9]"
re78 = function() paste0(seven_digits_regexp(), "|", eight_digits_regexp()) 
  
pmid_inds = function(x) grep(re78(), x)
  
isolate_pmid = function(x) {
   stopifnot(is.atomic(x), length(x)==1)
   cand = strsplit(x, " |: |\\)|:|\\]|;|'|\\(|\\[")[[1]]
   cand[grep(re78(), cand)]
  }
  
fabricate_anchor = function(x)
   sprintf("<a href=http://pubmed.gov/%s>%s</a>", x, x)
  
ancize_column = function(x) {
  comi = pmid_inds(x)
  for (i in comi) {
    jj = x[i]
    kk = sapply(jj, isolate_pmid)
    repl = sapply(kk, fabricate_anchor)
    for (j in seq_along(kk))
      x[i] = gsub(kk[j], repl[j], x[i])
    }
    x
  }
  
#' check columns of a dataframe for numerical tokens of 7 or 8 digits
#' and create HTML anchors to pubmed.gov constituting a link to a PMID
#' @param dataframe a data.frame instance
#' @return data.frame with HTML anchors to pubmed.gov inserted where 7- or 8-digit numbers are found
#' @note The method of isolating putative PMIDs is peculiar to patterns found in
#' the comment fields of annotated TF table (supplemental table S1 found in
#' \url{https://www.cell.com/cms/10.1016/j.cell.2018.01.029/attachment/88c0eca1-66f9-4068-b02e-bd3d55144f79/mmc2.xlsx} of PMID 29425488).  When DT::datatable is called on the output
#' of this function with `escape=FALSE` the PMIDs will render as hyperlinks.
#' Note that column 1 is assumed to be an ENSEMBL ID which could have 7 or 8 digits but is handled differently
#' @examples
#' litdf = data.frame(id="ENSG00000116819", a="Binds the same GCCTGAGGC sequence as the other AP-2s (PMID: 24789576)",
#'      stringsAsFactors=FALSE)
#' anchor_pmids(litdf)
#' @export
anchor_pmids = function(dataframe) {
  haspmid = sapply(dataframe, function(x) length(grep(re78(), x))>0)
  haspmid[1] = FALSE  # don't do ensembl IDs
  for (i in 1:ncol(dataframe)) {
   if (haspmid[i]) dataframe[,i] = ancize_column(dataframe[,i])
  }
  dataframe[,1] = sprintf("<A HREF='https://www.ensembl.org/Homo_sapiens/Gene/Summary?db=core;g=%s'>%s</A>",
        dataframe[,1], dataframe[,1])
  dataframe
}

library(TFutils)
data(tftColl)
nmsig = names(tftColl)
alltok = strsplit(nmsig, "_") 
allhgnc = AnnotationDbi::keys(org.Hs.eg.db::org.Hs.eg.db, keytype="SYMBOL")
allal = AnnotationDbi::keys(org.Hs.eg.db::org.Hs.eg.db, keytype="ALIAS")
alsym = AnnotationDbi::mapIds(org.Hs.eg.db::org.Hs.eg.db,
  keys=allal, column="SYMBOL", keytype="ALIAS")
names(alsym) = allal
names(allhgnc) = allhgnc
alltargs = c(allhgnc, alsym)
chk2 = sapply(alltok, function(x) sapply(x, function(z) 
  { 
  ans = try(alltargs[which(names(alltargs)==z)])
  if (inherits(ans, "try-error")) ans = NA_character_
  ans
  }))
newmap = cbind(tftname=names(tftColl), hgnc.heur=sapply(chk2, "[", 1), hgnc.fixed="no") 
# then edited by hand
# then mapped entries 556-615 using alis

# "https://genome.ucsc.edu/cgi-bin/hgTables?hgsid=686726561_9aeujyhQXuUhZyNoDYHyRzxA6TPa&clade=mammal&org=Human&db=hg19&hgta_group=allTables&hgta_track=hg19&hgta_table=metaDb&hgta_regionType=genome&position=chr21%3A33%2C031%2C597-33%2C041%2C570&hgta_outputType=primaryTable&hgta_outFileName=hgtab.txt"
library(data.table)
x = fread("hgtab.txt")
xdf = as.data.frame(x)
library(dplyr)
library(magrittr)
names(xdf)[1] = "obj"
library(tibble)
as.tibble(xdf) %>% filter(var %in% c("cell")) -> celtab
as.tibble(xdf) %>% filter(var %in% c("target")) -> tartab
library(AnnotationHub)
ah = AnnotationHub()
mah = mcols(ah)
mah = cbind(mah, AHID = rownames(mah))
tmah = as.tibble(mah)
trunctitle = gsub("\\..*", "", tmah[,1][[1]])
tmah$obj = trunctitle
inner_join(celtab, tartab, by="obj") -> ct
names(ct) = c("obj", "class1", "cell", "class2", "target")
inner_join(tmah, ct, by="obj") -> mk690
library(S4Vectors)
encode690 = DataFrame(mk690)
metadata(encode690) = list(source="https://genome.ucsc.edu/cgi-bin/hgTables?hgsid=686726561_9aeujyhQXuUhZyNoDYHyRzxA6TPa&clade=mammal&org=Human&db=hg19&hgta_group=allTables&hgta_track=hg19&hgta_table=metaDb&hgta_regionType=genome&position=chr21%3A33%2C031%2C597-33%2C041%2C570&hgta_outputType=primaryTable&hgta_outFileName=hgtab.txt", date=date())
save(encode690, file="encode690.rda")

#' process a gene_attribute_matrix.txt file from harmonizeome into a GeneSetCollection
#' @param txtfn character(1) path to gene_attribute_matrix.txt file from harmonizeome
#' @param tag character(1) will be added to shortDescription field of each GeneSet instance
#' @return GSEABase::GeneSetCollection
#' @note After uncompressing content of \url{http://amp.pharm.mssm.edu/static/hdfs/harmonizome/data/cheappi/gene_attribute_matrix.txt.gz} run this on gene_attribute_matrix.txt with tag="CHEA".
setupHIZE = function(txtfn="gene_attribute_matrix.txt", tag) {
# http://amp.pharm.mssm.edu/static/hdfs/harmonizome/data/cheappi/gene_attribute_matrix.txt.gz
 dd = read.delim(txtfn, sep="\t", skip=3, h=FALSE, stringsAsFactors=FALSE)
 dat = data.matrix(dd[,-c(1:3)])
 x = readLines(txtfn, n=3)
 tfn = strsplit(x[1], "\\t")[[1]]
 tfna = tfn[-c(1:3)]
 rownames(dat) = dd[,1]
 colnames(dat) = tfna
 targs = lapply(tfna, function(x) rownames(dat)[which(dat[,x]==1)]) 
 names(targs) = tfna
 cheaTargs = targs
 ch = lapply(cheaTargs, function(x) {
    tmp = GSEABase::GeneSet(GSEABase::SymbolIdentifier(), geneIds=x)
    tmp@shortDescription = paste(
      tmp@shortDescription, "(from", tag, ")")
    tmp
    })
 ch2 = lapply(names(ch), function(x){ GSEABase::setName(ch[[x]])=x; ch[[x]]})
 GSEABase::GeneSetCollection(ch2)
}

## ----setup,echo=FALSE---------------------------------------------------------
suppressWarnings({
suppressMessages({
suppressPackageStartupMessages({
library(BiocStyle)
library(TFutils)
library(org.Hs.eg.db)
library(GenomicFiles)
library(GO.db)
library(data.table)
library(knitr)
library(ggplot2)
library(data.table)
library(SummarizedExperiment)
library(BiocParallel)
library(png)
library(grid)
library(GSEABase) # for tftColl
library(gwascat)
data(fimoMap)
library(dplyr)
library(magrittr)
})
})
})

## ----setup2, echo=FALSE-------------------------------------------------------
library(TFutils)
library(AnnotationDbi)
suppressMessages({
tfdf = AnnotationDbi::select(org.Hs.eg.db::org.Hs.eg.db, 
    keys="GO:0003700", keytype="GO", 
    columns=c("ENTREZID", "SYMBOL"))
})
tfdf = tfdf[, c("ENTREZID", "SYMBOL")]
TFs_GO = TFCatalog(name="GO.0003700", nativeIds=tfdf$ENTREZID,
 HGNCmap=tfdf)

data(tftColl)
data(tftCollMap)
TFs_MSIG = TFCatalog(name="MsigDb.TFT", nativeIds=names(tftColl),
 HGNCmap=data.frame(tftCollMap,stringsAsFactors=FALSE))

data(cisbpTFcat)
TFs_CISBP = TFCatalog(name="CISBP.info", nativeIds=cisbpTFcat[,1],
 HGNCmap = cisbpTFcat)

data(hocomoco.mono.sep2018)
TFs_HOCO = TFCatalog(name="hocomoco11", nativeIds=hocomoco.mono.sep2018[,1],
 HGNCmap=hocomoco.mono.sep2018)

## ----lkupset,echo=FALSE,out.width='60%', fig.cap='Sizes of TF catalogs and of intersections based on HGNC symbols for TFs.', fig.pos='h'----
suppressPackageStartupMessages({library(UpSetR)})
allhg = keys(org.Hs.eg.db::org.Hs.eg.db, keytype="SYMBOL")
#activesym = unique(unlist(list(TFs_GO@HGNCmap[,2], TFs_HOCO@HGNCmap[,2], TFs_MSIG@HGNCmap[,2], TFs_CISBP@HGNCmap[,2])))
activesym = unique(unlist(list(HGNCmap(TFs_GO)[,2], HGNCmap(TFs_HOCO)[,2], HGNCmap(TFs_MSIG)[,2], HGNCmap(TFs_CISBP)[,2])))
use = intersect(allhg, activesym)
mymat = matrix(0, nr=length(use), nc=4)
rownames(mymat) = use
iu = function(x) intersect(x,use)
mymat[na.omit(iu(HGNCmap(TFs_GO)[,2])),1] = 1
mymat[na.omit(iu(HGNCmap(TFs_MSIG)[,2])),2] = 1
mymat[na.omit(iu(HGNCmap(TFs_HOCO)[,2])),3] = 1
mymat[na.omit(iu(HGNCmap(TFs_CISBP)[,2])),4] = 1
colnames(mymat) = c("GO", "MSigDb", "HOCO", "CISBP")
upset(data.frame(mymat),nsets=4,sets=c("MSigDb", "HOCO", "GO", "CISBP"), keep.order=TRUE, order.by="degree"
)

## ----TFclass, out.width='110%', fig.cap = 'Screenshots of AmiGO and TFClass hierarchy excerpts.',echo=FALSE----
knitr::include_graphics('AMIGOplus.png')

## ----dodo,echo=FALSE----------------------------------------------------------
library(knitr)
cismap = HGNCmap(TFs_CISBP)
scis = split(cismap, cismap$HGNC)
uf = vapply(scis, function(x) x$Family_Name[1],"character")
CISTOP = sort(table(uf),decreasing=TRUE)[1:10]
hoc = HGNCmap(TFs_HOCO)
shoc = split(hoc, hoc$HGNC)
sfam = vapply(shoc, function(x)x$`TF family`[1], "character")
HOTOP = sort(table(sfam),decreasing=TRUE)[1:10]
kable(data.frame(CISBP=names(CISTOP), Nc=as.numeric(CISTOP), 
   HOCOMOCO=names(HOTOP), Nh=as.numeric(HOTOP)), format="markdown")

## ----lkbro--------------------------------------------------------------------
TFutils::tftColl

## ----lktft2-------------------------------------------------------------------
grep("NFK", names(TFutils::tftColl), value=TRUE)

## ----demoredu-----------------------------------------------------------------
library(GenomicFiles)
data(fimo16)
fimo16
head(colData(fimo16))

## ----demoredu2, cache=TRUE----------------------------------------------------
if (.Platform$OS.type != "windows") {
 library(BiocParallel)
 register(SerialParam()) # important for macosx?
 rowRanges(fimo16) = GRanges("chr17", IRanges(38.077e6, 38.084e6))
 rr = GenomicFiles::reduceByFile(fimo16, MAP=function(r,f)
   scanTabix(f, param=r))
} else {
rr = readRDS(system.file("fimoser/fimo16rr.rds", package="TFutils"))
}

## ----paraa--------------------------------------------------------------------
asdf = function(x) data.table::fread(paste0(x, collapse="\n"), header=FALSE)
gg = lapply(rr, function(x) {
       tmp = asdf(x[[1]][[1]]) 
       data.frame(loc=tmp$V2, score=-log10(tmp$V7))
     })
for (i in 1:length(gg))  gg[[i]]$tf = colData(fimo16)[i,2]

## ----domat--------------------------------------------------------------------
matchcis = match(colData(fimo16)[,2], cisbpTFcat[,2])
famn = cisbpTFcat[matchcis,]$Family_Name
for (i in 1:length(gg))  gg[[i]]$tffam = famn[i]
nn = do.call(rbind, gg)

## ----finish, fig.height=3.5,echo=FALSE,fig.cap='TF binding in the vicinity of gene ORMDL3.  Points are -log10-transformed FIMO-based p-values colored according to TF class as annotated in CISBP.  Segments at bottom of plot are transcribed regions of ORMDL3 according to UCSC gene models in build hg19.'----
library(ggplot2)
myf = function(a=38077296, b=38078938) 
  geom_segment(aes(x=a, xend=b, y=2.85, yend=2.85, 
  colour="[ORMDL3]"))
ggplot(nn, aes(x=loc,y=score,group=tffam, colour=tffam)) + 
  geom_point() + myf() + myf(38079365, 38079516) + 
    myf(38080283, 38080478) + myf(38081008, 38081058) + 
    myf(38081422, 38081624) + myf(38081876, 38083094) + 
    myf(38083737, 38083884) + ylab("-log10 FIMO p-value") + 
 xlab("position on chr17, hg19")

## ----lkbi, fig.cap='Binding of CEBPB in the vicinity of ORMDL3 derived from ChIP-seq experiments in four cell lines reported by ENCODE.  Colored rectangles at top are regions identified as narrow binding peaks, arrows in bottom half are exons in ORMDL3.  Arrows sharing a common vertical position are members of the same transcript as cataloged in Ensembl version 75.', echo=FALSE----
pp = readPNG("ormdl3CEBPB.png")
grid.raster(pp)

## ----lktarapp, fig.cap='TFtargs() screenshot.  This example reports on recent EBI GWAS catalog hits on chromosome 17 only.',echo=FALSE----
pp = readPNG("tfTargsApp.png")
grid.raster(pp)

## ----ljlj---------------------------------------------------------------------
data(cisbpTFcat)
TFs_CISBP = TFCatalog(name="CISBP.info", 
   nativeIds=cisbpTFcat[,1],
   HGNCmap = cisbpTFcat)
TFs_CISBP

## ----defdir-------------------------------------------------------------------
library(dplyr)
library(magrittr)
library(gwascat)
#data(ebicat37)
load(system.file("legacy/ebicat37.rda", package="gwascat"))
directHitsInCISBP("Rheumatoid arthritis", ebicat37)

## ----lktr---------------------------------------------------------------------
tt = topTraitsOfTargets("MTF1", TFutils::tftColl, ebicat37)
head(tt)
table(tt[,1])

## ----dose---------------------------------------------------------------------
sessionInfo()

#' set up list of pfms in motifStack protocol
#' @param tffam character(1) name of TF family as found in TFutils::hocomoco.mono field `TF family`
#' @param trimfac fraction passed as parameter `t` to motifStack::trimMotif
#' @note Uses MotifDb, motifStack to create a list of pfms
#' @return a list of pfm instances as defined in motifStack
#' @examples
#' n1 = tffamCirc.prep()
#' str(n1)
#' @export
tffamCirc.prep = function(tffam = "Paired-related HD factors{3.1.3}",
    trimfac = 0.4) {
  chkns = function(p) {
   if (!requireNamespace(p)) 
      stop(sprintf("please install package %s to use this function.", p))
   }
  chk = sapply(c("motifStack", "MotifDb", "dplyr", "magrittr", 
              "RColorBrewer"), chkns)
  "%>%" = magrittr::"%>%"
  prhdvec = dplyr::mutate(TFutils::hocomoco.mono, TFfamily=`TF family`) %>% 
   dplyr::filter(TFfamily == "Paired-related HD factors{3.1.3}") %>% 
   dplyr::select(`Transcription factor`) %>% unlist %>% unname
  #
  #
  cands = lapply(prhdvec, function(x) MotifDb::query(MotifDb::MotifDb, c(x, "jaspar2018", "Hsapiens")))
  ok = sapply(cands, function(x) length(x)>0)
  cands = cands[ok]
  mots = lapply(cands, function(x) as.list(x)[[1]]) # just pick first when multiple
  # pfm class is accessible with requireNamespace
  mots = lapply(1:length(mots), function(x) new("pfm", mots[[x]], name=prhdvec[ok][x]))
  motifs2 = mots
  if (trimfac>0) motifs2 = lapply(mots, motifStack::trimMotif, t=trimfac)
  names(motifs2) = prhdvec[ok]
  motifs2
}

#' use a radial plot (by default) for motif stack
#' @param motiflist a list of pfm instances from motifStack
#' @param circosParms a list of parameter settings for circos plot
#' @return side effect to graphics device
#' @examples
#' p1 = tffamCirc.prep( )
#' tffamCirc.plot(p1[c(1:8, 10:17, 19)])
#' @export
tffamCirc.plot = function(motiflist, circosParms = defaultCircosParms()) {
  ### plot logo stack with radial style
  ans = list(pfms = motiflist)
  ans = c(ans, circosParms)
  do.call(motifStack::motifStack, ans)
}
     
#' basic layout parameters for circos
#' @return a list
#' @examples
#' head(defaultCircosParms())
#' @export
defaultCircosParms = function() {
        color <- RColorBrewer::brewer.pal(12, "Set3")
        list(layout="radialPhylog", 
             circle=0.3, cleaves = 0.2, 
             clabel.leaves = 0.5, 
             col.bg=rep(color, each=5), col.bg.alpha=0.3, 
             col.leaves=rep(color, each=5),
             col.inner.label.circle=rep(color, each=5), 
             inner.label.circle.width=0.05,
             col.outer.label.circle=rep(color, each=5), 
             outer.label.circle.width=0.02, 
             circle.motif=1.2,
             angle=350)
        }

#  motifStack::motifStack(motifs2[c(1:8,10:17,19)], layout="radialPhylog", 

#' define a structure to hold information about TFs from diverse reference sources
#' @importFrom methods new show
#' @importFrom stats na.omit
#' @slot name character 
#' @slot nativeIds character tokens used by the provider to enumerate transcription factors
#' @slot HGNCmap data.frame with atleast two columns,
#' native id as first column and HGNC symbol as second column
#' @slot metadata ANY
#' @note This class respects the notions that 1) a source of information
#' about transcription factors should have a name, 2) each source
#' has its own 'native' nomenclature for the factors themselves,
#' 3) it is common to use the gene symbol to refer to the transctiption
#' factor, and 4) additional metadata will frequently be required to
#' establish information about provenance of assertions about transcription
#' factors.
#' @aliases "TFCatalog-class"
#' @export
setClass("TFCatalog", representation(name="character",
                                     nativeIds="character", HGNCmap="data.frame", metadata="ANY"))
#' Constructor for TFCatalog
#' @param name informative character(1) for collection
#' @param nativeIds character() vector of identifiers used by collection creators
#' @param HGNCmap data.frame with column 1 nativeIds, column 2 HGNC or hgnc.heur for MSigDb
#' and any other columns of use
#' @param metadata a list of metadata elements
#' @return instance of TFCatalog
#' @examples
#' if (require("GSEABase")) {
#'  TFs_MSIG = TFCatalog(name="MsigDb.TFT",nativeIds=names(TFutils::tftColl),
#'  HGNCmap=data.frame(TFutils::tftCollMap,stringAsFactors=FALSE))
#'  TFs_MSIG
#' }
#' @export
TFCatalog = function(name, nativeIds, HGNCmap, metadata) {
  if (missing(metadata)) metadata=list()
  new("TFCatalog", name=name, nativeIds=nativeIds,
      HGNCmap=HGNCmap, metadata=metadata)
}
#' simple accessor for HGNCmap component of TFCatalog
#' @importFrom methods slot
#' @param x instance of TFCatalog
#' @return dataframe instance
#' @examples
#' HGNCmap
#' @export
HGNCmap = function(x) slot(x, "HGNCmap")
#' produce a concise report on TFCatalog instance
#' @aliases show,TFCatalog-method
#' @return side effect
#' @param object instance of TFCatalog
#' @export
setMethod("show", "TFCatalog", function(object) {
  reqNS("Biobase")
  cat("TFutils TFCatalog instance", object@name, "\n")
  cat(sprintf(" %d native Ids, including\n   ", length(object@nativeIds)))
  cat(Biobase::selectSome(object@nativeIds, max=2), "\n")
  cat(sprintf(" %d unique HGNC tags, including\n   ", length(unique(object@HGNCmap[,2]))))
  cat(Biobase::selectSome(na.omit(object@HGNCmap[,2])), "\n")
})

#' demonstrate interoperation of TF catalog with GWAS catalog
#' @param traitTag character(1) string found in DISEASE/TRAIT field of gwascat instance
#' @param gwascat instance of \code{\link[gwascat]{gwaswloc-class}}
#' @return data.frame 
#' @examples
#' data(gwascat_hg19_chr17)
#' directHitsInCISBP("Prostate cancer" , gwascat_hg19_chr17)
#' @export
directHitsInCISBP = function(traitTag, gwascat) {
   reqNS("S4Vectors")
   reqNS("dplyr")
   stopifnot(traitTag %in% gwascat$`DISEASE/TRAIT`)
   as.data.frame(S4Vectors::mcols(gwascat)) %>%  # names are normalized
     dplyr::select(DISEASE.TRAIT, MAPPED_GENE) %>%
     dplyr::filter(DISEASE.TRAIT==traitTag) %>%
     dplyr::mutate(HGNC=MAPPED_GENE) %>%
     dplyr::inner_join(cisbpTFcat) %>%
     dplyr::select(HGNC, Family_Name) %>% unique()
   }
symToTFTName = function(TFsym, indToUse=1) {
  tfmap = TFutils::tftCollMap
  ind = which(tfmap$hgnc.heur == TFsym)
  if (length(ind)==0) stop(paste("could not find", TFsym, "in tftCollMap"))
  if (length(ind)>1) {
        warning(paste("found more than one heuristic match to",
             TFsym, "be sure indToUse is properly set"))
        message(paste("using", tfmap[ind[indToUse],1]))
        }   
  tfmap[ind[indToUse],1]
}
#' Use MSigDB TF targets resource to find targets of input TF and find traits to which these targets have been mapped
#' @importFrom GSEABase geneIds SymbolIdentifier mapIdentifiers geneIdType
#' @param TFsym character(1) symbol for a TF must be present in \code{tftCollMap[, "hgnc.heur"]}
#' @param gsc an instance of \code{\link[GSEABase]{GeneSetCollection-class}}, intended to enumerate
#' targets of a single transcription factor in each GeneSet, as in TFutils::tftColl
#' @param gwcat instance of \code{\link[gwascat]{gwaswloc-class}}
#' @param ntraits numeric(1) number of traits to report
#' @param force logical see note, set to true if you want to skip mapping from TFsym to a specific motif or TF identifier used as name of a GeneSet in gsc
#' @param \dots character() vector of fields in mcols(gwcat) to include
#' @note If \code{tftCollMap[, "hgnc.heur"]} does not possess the necessary
#' @return data.frame
#' symbol, set force = TRUE to use a known 'motif' name among `names(gsc)`
#' @examples
#' suppressPackageStartupMessages({
#' library(GSEABase)
#' })  # more results if you substitute ebicat37 from gwascat below
#' topTraitsOfTargets("MTF1" , tftColl, gwascat_hg19_chr17)
#' @export
topTraitsOfTargets = function(TFsym, gsc, gwcat, ntraits=6, force=FALSE, ...) {
  if (geneIdType(gsc[[1]])@type != "Symbol") {
   message("remapping identifiers of input GeneSetCollection to Symbol...")
   gsc = mapIdentifiers(gsc, SymbolIdentifier("org.Hs.eg.db"))
   message("done")
   }   
  collToUse = symToTFTName(TFsym, ...)
  targs = intersect(geneIds(gsc[[collToUse]]), gwcat$MAPPED_GENE)
  intinds = which(gwcat$MAPPED_GENE %in% targs)

  ans = mcols(gwcat[intinds])[, c("DISEASE/TRAIT", "MAPPED_GENE",
       "SNPS", "CHR_ID", "CHR_POS", ...)]
  trtab = table(glis <- gwcat$`DISEASE/TRAIT`[which(gwcat$MAPPED_GENE %in% targs)])
  topl = sort(trtab, decreasing=TRUE)[seq_len(ntraits)]
  ans = ans[which(ans[,1] %in% names(topl)),]
  as.data.frame(ans) %>% arrange(DISEASE.TRAIT) %>% unique()
}

#' gwascat_hg19: GRanges of march 21 2018 EBI gwascat, limit to chr17
#' @docType data
#' @format GenomicRanges GRanges instance
#' @source gwascat::makeCurrentGwascat, with gwascat:::lo38to19 applied
#' @examples
#' TFutils::gwascat_hg19_chr17[,1:5]
"gwascat_hg19_chr17"

#' fimoMap: table with Mnnnn (motif PWM tags) and HGNC symbols for TFs
#' @docType data
#' @format data.frame
#' @source Kimberly Glass (rekrg@channing.harvard.edu) 
#' @examples
#' head(TFutils::fimoMap)
"fimoMap"

#' tftColl: GSEABase GeneSetCollection for transcription factor targets
#' @importClassesFrom GSEABase GeneSetCollection
#' @docType data
#' @format GSEABase GeneSetCollection instance
#' @source broad institute
#' @note run GSEABase::getGMT() on c3/TFT geneset collection
#' from MSigDb
#' @examples
#' TFutils::tftColl
"tftColl"

#' fimo16: GenomicFiles instance to AWS S3-resident FIMO bed for 16 TFs
#' @docType data
#' @format GenomicFiles for a TabixFileList
#' @source K. Glass FIMO runs, see \url{https://doi.org/10.1016/j.celrep.2017.10.001}
#' @examples
#' TFutils::fimo16
"fimo16"

#' cisbpTFcat: data.frame with information on CISBP TFs for human, retained for reproducibility support; see
#' cisbpTFcat_2.0 for a more recent catalog
#' @docType data
#' @format data.frame
#' @source \url{http://cisbp.ccbr.utoronto.ca/bulk.php} select Homo_sapiens
#' @note Extracted March 2018, checked August 2018.  The only changes
#' observed are that genes ZUFSP and T are used has HGNC values
#' in the March catalog; these symbols seem to be absent from the 
#' org.Hs.eg.db of August 2018.  The records involved are 
#' 1356, 7412 and 7413.  These symbols were left in the package image
#' of CISBP in August 2018.
#' @examples
#' head(TFutils::cisbpTFcat)
"cisbpTFcat"

#' hocomoco.mono: data.frame with information on HOCOMOCO TFs for human
#' @docType data
#' @format data.frame
#' @source \url{http://hocomoco11.autosome.ru/human/mono?full=true} 
#' @note Extracted March 2018
#' @examples
#' head(TFutils::hocomoco.mono)
"hocomoco.mono"

#' tftCollMap: data.frame with information on MSigDb TFs for human
#' @docType data
#' @format data.frame
#' @source \url{http://software.broadinstitute.org/gsea/msigdb/genesets.jsp?collection=TFT}
#' @note Annotation of TFs is ad-hoc.  GeneSet names were tokenized, splitting
#' by underscore, and then fragments were matched to SYMBOL and ALIAS elements
#' of org.Hs.eg.db.  Extracted March 2018
#' @examples
#' head(TFutils::tftCollMap)
"tftCollMap"

#' metadata_tf: list with metadata (motif_if and hgnc_symbol) about all the CISBP FIMO scan TF bed files
#' @docType data
#' @format list
#' @source K. Glass ran FIMO
#' @examples
#' TFutils::metadata_tf
"metadata_tf"

#' named_tf: named list with the names being the hgnc_symbol of the motif_id
#' @docType data
#' @format list
#' @source K. Glass ran FIMO
#' @examples
#' TFutils::named_tf
#' named_tf[["VDR"]]
"named_tf"

#' tfhash: data.frame with MSigDb TFs, TF targets as symbol or ENTREZ
#' @docType data
#' @format list
#' @source MSigDb "c3" (motif gene sets) has been harvested for simple annotation of TFs and targets.
#' @examples
#' TFutils::tfhash
#' tfhash[1:3,]
"tfhash"

#' encode690: DataFrame extending AnnotationHub metadata about ENCODE cell line x TF ranges
#' @docType data
#' @format DataFrame
#' @source see metadata(encode690)
#' @examples
#' names(TFutils::encode690)
#' TFutils::encode690[,1:5]
"encode690"

#' hocomoco.mono.sep2018: data.frame with information on HOCOMOCO TFs for human, Sept 2018 download
#' @docType data
#' @format data.frame
#' @source \url{http://hocomoco11.autosome.ru/human/mono?full=true} 
#' @note Extracted September 2018
#' @examples
#' head(TFutils::hocomoco.mono.sep2018)
"hocomoco.mono.sep2018"

#' a list of GRanges instances with TF FIMO scores returned by `fimo_granges`
#' @docType data 
#' @format a list of GRanges instances
#' @examples
#' names(S4Vectors::mcols(demo_fimo_granges$VDR[[1]]))
"demo_fimo_granges"

#' a Seqinfo instance for a chr17 in hg19
#' @docType data 
#' @format a Seqinfo instance
#' @examples
#' seqinfo_hg19_chr17
"seqinfo_hg19_chr17"

#' cisbpTFcat_2.0: data.frame with information on CISBP TFs for human, described in PMID 31133749
#' @docType data
#' @format data.frame
#' @source \url{http://cisbp.ccbr.utoronto.ca/bulk.php} select Homo_sapiens
#' @note Extracted August 2019.
#' @examples
#' head(TFutils::cisbpTFcat_2.0)
"cisbpTFcat_2.0"

#' lambert_snps is Table S3 of Lambert et al PMID 29425488
#' @docType data
#' @format data.frame
#' @examples
#' head(lambert_snps)
"lambert_snps"

## ----lkfimo16-----------------------------------------------------------------
suppressPackageStartupMessages({
library(TFutils)
library(GenomicRanges)
})
fimo16

## ----lkimp--------------------------------------------------------------------
if (.Platform$OS.type != "windows") {
 si = TFutils::seqinfo_hg19_chr17
 myg = GRanges("chr17", IRanges(38.07e6,38.09e6), seqinfo=si)
 colnames(fimo16) = fimo16$HGNC 
 lk2 = GenomicFiles::reduceByRange(fimo16[, c("POU2F1", "VDR")],
   MAP=function(r,f) scanTabix(f, param=r))
 str(lk2)
}

## ----domult-------------------------------------------------------------------
#fimo_ranges = function(gf, query) { # prototypical code
# rowRanges(gf) = query
# ans = GenomicFiles::reduceByRange(gf, MAP=function(r,f) scanTabix(f, param=r))
# ans = unlist(ans, recursive=FALSE)  # drop top list structure
# tabs = lapply(ans, lapply, function(x) {
#     con = textConnection(x)
#     on.exit(close(con))
#     dtf = read.delim(con, h=FALSE, stringsAsFactors=FALSE, sep="\t")
#     colnames(dtf) = c("chr", "start", "end", "rname", "score", "dir", "pval")
#     ans = with(dtf, GRanges(seqnames=chr, IRanges(start, end),
#            rname=rname, score=score, dir=dir, pval=pval))
#     ans
#     })
# GRangesList(unlist(tabs, recursive=FALSE))
#}
if (.Platform$OS.type != "windows") {
 rr = fimo_granges(fimo16[, c("POU2F1", "VDR")], myg)
 rr
}

## ----sesss--------------------------------------------------------------------
sessionInfo()

library(ontoProc)
clo = getCellLineOnto()
encode690$lcell = tolower(encode690$cell)
clomat = clo$name[ which(gsub(" cell", "", tolower(clo$name)) %in% 
   intersect(tolower(encode690$cell), gsub(" cell", "", tolower(clo$name))))]
clom = data.frame(tag=names(clomat), name=as.character(clomat))
rownames(clom) = make.names(tolower(gsub(" cell", "", clom$name)), unique=TRUE)
clom[encode690$lcell,] -> dd
encode690$tag = as.character(dd$tag)
encode690$CLOname = as.character(dd$name)

#' create table of TF targets and related metadata
#' @import dplyr
#' @import magrittr
#' @importFrom methods as is
#' @param tfstub character(1) gene-like symbol for TF; will be grepped in names(gscoll)
#' @param gscoll a GSEABase GeneSetCollection
#' @param orgdb an instance of OrgDb as defined in AnnotationDbi
#' @param gwrngs a GRanges representing EBI gwascat, must have `DISEASE/TRAIT`, `MAPPED_GENE`
#' @return data.frame instance
#' @note This function will link together information on targets of a
#' given TF to the GWAS catalog.
#' @examples
#' gt = grabTab("VDR", gscoll=TFutils::tftColl,
#'    orgdb=org.Hs.eg.db::org.Hs.eg.db, gwrngs=TFutils::gwascat_hg19_chr17)
#' dim(gt)
#' head(gt)
#' @export
grabTab = function(tfstub="STAT1", gscoll=TFutils::tftColl, 
     orgdb=org.Hs.eg.db::org.Hs.eg.db, gwrngs=TFutils::gwascat_hg19_chr17) {
reqNS("S4Vectors")
MAPPED_GENE <- CHR_ID <- `DISEASE/TRAIT` <- NULL
CHR_POS <- REGION <- NULL
ngs = sapply(gscoll, GSEABase::setName)
allst1 = unlist(lapply(gscoll[ grep(tfstub, ngs) ], GSEABase::geneIds))
st1syms = AnnotationDbi::mapIds(orgdb, keys=allst1, keytype="ENTREZID", column="SYMBOL")
chk = as(S4Vectors::mcols(gwrngs), "data.frame") %>% 
     dplyr::filter(MAPPED_GENE %in% st1syms) %>% dplyr::select(`DISEASE/TRAIT`, MAPPED_GENE, CHR_ID, CHR_POS, REGION)
cbind(TF=tfstub, chk)
}

# Update Oct 31 2020?
cell_url = function() "https://www.cell.com/cms/10.1016/j.cell.2018.01.029/attachment/ede37821-fd6f-41b7-9a0e-9d5410855ae6/mmc2.xlsx"

#' acquire the Excel spreadsheet content for table S1 of Lambert et al. Cell 2018, "The Human Transcription Factors"
#' @import BiocFileCache
#' @import readxl
#' @param cache a BiocFileCache instance
#' @return a tbl_df
#' @note This will download the spreadsheet if not found in `cache`.
#' @examples
#' if (interactive()) retrieve_lambert_main()
#' @export
retrieve_lambert_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
 uuid_element = "ede37821-fd6f"
 peek = BiocFileCache::bfcquery(cache, uuid_element)
 if(length(peek$rpath)==0) 
   BiocFileCache::bfcadd(cache, 
    cell_url())
 peek = try(BiocFileCache::bfcquery(cache, uuid_element))
 if (inherits(peek, "try-error") || length(peek$rpath)==0) stop('could not retrieve xlsx')
 ans = suppressMessages(suppressWarnings(readxl::read_xlsx(peek$rpath, skip=1, sheet=2)))
 names(ans)[4] = "Is TF?"
 ans
}

#' use DT::datatable to browse the Lambert table S1
#' @param cache a BiocFileCache instance
#' @note PMIDs are converted to HTML anchors and DT::datatable is run with `escape=FALSE`.
#' @return result of DT::datatable
#' @examples
#' if (interactive()) browse_lambert_main()
#' @export
browse_lambert_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
 tab = as.data.frame(retrieve_lambert_main(cache=cache))
 tab = anchor_pmids(tab)
 DT::datatable(tab, escape=FALSE)
}

#
# task 1: given a conventional name for a TF, return
# addresses of putative binding sites (as recorded using FIMO)
# in a GRanges.  you must define a source and a genomic region
# for 
#

#' utility to generate link to biocfound bucket for FIMO TFBS scores
#' @param tag character(1) token identifying TF, can be an HGNC gene name or Mnnnn PWM tag.
#' It must be findable in TFutils::fimoMap table.
#' @return character(1) URL
#' @examples
#' URL_s3_tf
#' @export
URL_s3_tf = function(tag="M3433") {
 tab = TFutils::fimoMap
 pwmind = which(tab[,"PWMid"] == tag)
 if (length(pwmind)==1)
  return(sprintf("http://s3.amazonaws.com/biocfound-tfbs/%s.02sort.bed.gz",
    tag))
 else gind = which(tab[,"HGNC"] == tag)
 if (length(gind)==1)
  return(sprintf("http://s3.amazonaws.com/biocfound-tfbs/%s.02sort.bed.gz",
    tab[gind,"PWMid"]))
 else stop("could not locate tag in TFutils::fimoMap")
 }

#' Update Oct 18 2021
ccbr_cell_url = function() "http://humantfs.ccbr.utoronto.ca/download/v_1.01/DatabaseExtract_v_1.01.csv"

#' acquire the CSV content for table S1 of Lambert et al. Cell 2018 from the Human TFS repository at http://humantfs.ccbr.utoronto.ca 
#' @import BiocFileCache
#' @import readxl
#' @importFrom utils read.csv2
#' @param cache a BiocFileCache instance
#' @return a tbl_df
#' @note This will download the spreadsheet if not found in `cache`.
#' @examples
#' if (interactive()) retrieve_humantfs_main()
#' @export
#' 
#' 
retrieve_humantfs_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
 uuid_element = "humantfs.ccbr.utoronto.ca/download/v_1.01/DatabaseExtract_v_1.01.csv"
 peek = BiocFileCache::bfcquery(cache, uuid_element)
 if(length(peek$rpath)==0) 
   BiocFileCache::bfcadd(cache, 
    ccbr_cell_url())
 peek = try(BiocFileCache::bfcquery(cache, uuid_element))
 if (inherits(peek, "try-error") || length(peek$rpath)==0) stop('could not retrieve csv')
 ans = suppressMessages(suppressWarnings(read.csv2(peek$rpath, header = TRUE, stringsAsFactors = FALSE, sep = ","))) 
 ans
}

    

#' use DT::datatable to browse the Lambert's Human Transcription Factors repository
#' @param cache a BiocFileCache instance
#' @return result of DT::datatable
#' @examples
#' if (interactive()) browse_humantfs_main()
#' @export
browse_humantfs_main = function(cache=BiocFileCache::BiocFileCache(ask=FALSE)) {
 tab = as.data.frame(retrieve_humantfs_main(cache=cache))
 tab = tab[ , -1] 
 idx = tab$Is.TF. == 'Yes' # keep only if TF = Yes 
 tab = tab[ idx, ]
 DT::datatable(tab, escape=FALSE)
}

library(TFutils)
data("named_tf")
fimo_bs = getBS("VDR", named_tf, GenomicRanges::GRanges("chr1", IRanges::IRanges(1,25000)),importFIMO_local)
library(feather)
library(dplyr)
library(magrittr)
library(data.table)
tfhash = read_feather(system.file("feather/tfhash2.feather", package = "TFutils"))
tfhash_vdr = tfhash %>% filter(tfname %like% 'VDR')
#Grab some target genes of VDR
library(TxDb.Hsapiens.UCSC.hg19.knownGene)
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
txdb
seqlevels(txdb) <- "chr1" #looking at chromosome 1 
keys = as.character(tfhash_vdr$targets) # these are geneids
#select(txdb, keys = keys, columns="TXNAME", keytype="GENEID")
#GR <- transcripts(txdb, filter=list(tx_chrom = "chr1"))
GR <- genes(txdb, filter=list(tx_chrom = "chr1"))
library(GenomicRanges)
subsetByOverlaps(GR, fimo_bs)

#' utility to obtain location etc. for rsids of SNPs
#' @param rsids character vector of dbSNP identifiers
#' @note Uses rest.ensembl.org, posting to variant_recorder/homo_sapiens.
#' Parses result minimally, using only the first SPDI to obtain
#' location information, adding 1 as ensembl genomic coordinates
#' are zero-based.
#' @return GRanges instance
#' @examples
#' if (interactive()) get_rslocs_38() # see https://stat.ethz.ch/pipermail/bioc-devel/2020-October/017263.html
#' @export
get_rslocs_38 = function(rsids = c("rs6060535", "rs56116432")) {
 server <- "https://rest.ensembl.org"
 ext <- "/variant_recoder/homo_sapiens"
 r <- httr::POST(paste(server, ext, sep = ""), 
       httr::content_type("application/json"), 
       httr::accept("application/json"), 
       body = list(ids=rsids), encode="json")
 httr::stop_for_status(r)
 ans = rjson::fromJSON( rjson::toJSON( httr::content(r)))
 ids = lapply(ans, "[[", "id")
 spd = sapply(lapply(ans, "[[", "spdi"), "[", 1)
 sspd = strsplit(spd, ":")
 chrs = sapply(sspd, "[", 1)
 locs = as.numeric(sapply(sspd, "[", 2))+1
 ans = GenomicRanges::GRanges(chrs, IRanges::IRanges(locs, width=1))
 Seqinfo::genome(ans) = "GRCh38"
 names(ans) = ids
 ans
}

#' browse several hundred disease-TF associations with hyperlinked PMIDs
#' @importFrom stats family
#' @importFrom utils read.csv read.delim
#' @examples
#' if (interactive()) browse_lambert_gwaslinks()
#' @return DT::datatable
#' @note Based on supplemental table S4 of PMID 29425488
#' @export
browse_lambert_gwaslinks = function() {
 dis = read.csv(system.file("lambert/S4_DiseaseTFMuts.csv", package=
   "TFutils"), 
   stringsAsFactors=FALSE, check.names=FALSE)
 dis[192,][,3] = gsub(", ", ",", dis[192,3])
 fabricate_anchor = function(x)
    sprintf("<a href=http://pubmed.gov/%s>%s</a>", x, x)
 pms = dis[,3]
 spms = strsplit(pms, ",")
 p0 = function(x) paste0(x, collapse=", ")
 dis$PMID = sapply(spms, function(z)p0(unname(sapply(z, function(x)fabricate_anchor(x)))))
 DT::datatable(dis, escape=FALSE)
}

---
title: "TFutils: Data Structures for Transcription Factor Bioinformatics"
author: 
  - name: BJ Stubbs
    affiliation: Channing Division of Network Medicine, Brigham and Women's Hospital, Harvard Medical School
  - name: Shweta Gopaulakrishnan
    affiliation: Channing Division of Network Medicine, Brigham and Women's Hospital, Harvard Medical School
  - name: Kimberly Glass
    affiliation: Channing Division of Network Medicine, Brigham and Women's Hospital, Harvard Medical School
  - name: Nathalie Pochet
    affiliation: Department of Neurology, Brigham and Women's Hospital, Harvard Medical School; Broad Institute
  - name: Celine Everaert
    affiliation: Department of Neurology, Brigham and Women's Hospital, Harvard Medical School; Broad Institute
  - name: Benjamin Raby
    affiliation: Channing Division of Network Medicine, Brigham and Women's Hospital, Harvard Medical School
  - name: Vincent Carey
    affiliation: Channing Division of Network Medicine, Brigham and Women's Hospital, Harvard Medical School
abstract: >
  DNA transcription is intrinsically complex.  Bioinformatic work 
  with transcription factors (TFs) is complicated by a multiplicity of data resources 
  and annotations.  The Bioconductor package _TFutils_ includes data structures 
  and functions to enhance the precision and utility of integrative analyses 
  that have components involving TFs.  _TFutils_ provides catalogs of human TFs
  from three reference sources (CISBP, HOCOMOCO, and GO), a catalog of
  TF targets derived from MSigDb, and multiple approaches to enumerating
  TF binding sites.  Aspects of integration of TF binding
  patterns and genome-wide association study results are explored in
  examples.
keywords: Bioinformatics, DNA transcription, Transcription factors
bibliography: tfsrcs.bib
includes:  
    in_header: preamble-latex.tex
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{TFutils -- representing TFBS and TF target sets}
  %\VignetteEncoding{UTF-8}
urlcolor: blue
---

# Introduction

```{r setup,echo=FALSE}
suppressWarnings({
suppressMessages({
suppressPackageStartupMessages({
library(BiocStyle)
library(TFutils)
library(org.Hs.eg.db)
library(GenomicFiles)
library(GO.db)
library(data.table)
library(knitr)
library(ggplot2)
library(data.table)
library(SummarizedExperiment)
library(BiocParallel)
library(png)
library(grid)
library(GSEABase) # for tftColl
library(gwascat)
data(fimoMap)
library(dplyr)
library(magrittr)
})
})
})
```

A central concern of genome biology is improving
understanding of gene transcription.  In simple
terms, transcription factors (TFs)
are proteins that bind to DNA, typically near gene promoter
regions.  The role of TFs in gene expression variation
is of great interest.  Progress in deciphering
genetic and epigenetic processes that affect TF abundance
and function will be essential in clarifying and
interpreting gene expression variation patterns
and their effects on phenotype.  Difficulties
of identifying functional
binding of TFs, and opportunities for using information
of TF binding in
systems biology contexts, are reviewed in @Lambert2018 and @Weirauch2014.

This paper describes an R/Bioconductor package called
TFutils, which assembles various resources intended
to clarify and unify approaches to working with
TF concepts in bioinformatic analysis.  Computations
described in this paper can be carried out with
Bioconductor version 3.8.  The package can be 
installed with
```
# use install.packages("BiocManager") if not already available
library(BiocManager) 
install("TFutils")
```

In the next section we describe the basic concepts of enumerating and
classifying TFs, enumerating TF targets, and representing
genome-wide quantification of TF binding affinity.
This is followed by a review of the key data
structures and functions provided in the package, and an
example in cancer informatics.

The present paper does not deal directly with the manipulation or
interpretation of sequence motifs.  An excellent Bioconductor
package that synthesizes many approaches to these tasks is
`r Biocpkg("universalmotif")`.

# Basic concepts of transcription factor bioinformatics

## Enumerating transcription factors

Given the importance of the topic, it is not surprising that
a number of bioinformatic research groups have published
catalogs of transcription factors along with metadata
about their features.  Standard nomenclature
for TFs has yet to be established.  Gene symbols,
motif sequences, and position-weight matrix catalog
entries have all been used as TF identifiers.

In TFutils we have gathered information from four
widely used resources, focusing specifically
on human TFs: Gene Ontology (GO, @Ashburner2000,
in which `GO:0003700` is the tag for the molecular
function concept "DNA binding transcription factor activity"),
CISBP (@Weirauch2014), HOCOMOCO (@Kulakovskiy2018),
and the "c3 TFT (transcription factor target)" signature set of MSigDb (@Subramanian15545).
Figure \@ref(fig:lkupset) depicts
the sizes of these
catalogs, measured using counts 
of unique HGNC gene symbols.
The enumeration for GO uses Bioconductor's
`r Biocpkg("org.Hs.eg.db")` package to
find direct associations from `GO:0003700`
to HGNC symbols.  The enumeration for
MSigDb is heuristic
and involves parsing the gene set identifiers
used in MSigDb
for exact or close matches to HGNC symbols.
For CISBP and HOCOMOCO, the associated web
servers provide easily parsed tabular catalogs.

```{r setup2, echo=FALSE}
library(TFutils)
library(AnnotationDbi)
suppressMessages({
tfdf = AnnotationDbi::select(org.Hs.eg.db::org.Hs.eg.db, 
    keys="GO:0003700", keytype="GO", 
    columns=c("ENTREZID", "SYMBOL"))
})
tfdf = tfdf[, c("ENTREZID", "SYMBOL")]
TFs_GO = TFCatalog(name="GO.0003700", nativeIds=tfdf$ENTREZID,
 HGNCmap=tfdf)

data(tftColl)
data(tftCollMap)
TFs_MSIG = TFCatalog(name="MsigDb.TFT", nativeIds=names(tftColl),
 HGNCmap=data.frame(tftCollMap,stringsAsFactors=FALSE))

data(cisbpTFcat)
TFs_CISBP = TFCatalog(name="CISBP.info", nativeIds=cisbpTFcat[,1],
 HGNCmap = cisbpTFcat)

data(hocomoco.mono.sep2018)
TFs_HOCO = TFCatalog(name="hocomoco11", nativeIds=hocomoco.mono.sep2018[,1],
 HGNCmap=hocomoco.mono.sep2018)
```

```{r lkupset,echo=FALSE,out.width='60%', fig.cap='Sizes of TF catalogs and of intersections based on HGNC symbols for TFs.', fig.pos='h'}
suppressPackageStartupMessages({library(UpSetR)})
allhg = keys(org.Hs.eg.db::org.Hs.eg.db, keytype="SYMBOL")
#activesym = unique(unlist(list(TFs_GO@HGNCmap[,2], TFs_HOCO@HGNCmap[,2], TFs_MSIG@HGNCmap[,2], TFs_CISBP@HGNCmap[,2])))
activesym = unique(unlist(list(HGNCmap(TFs_GO)[,2], HGNCmap(TFs_HOCO)[,2], HGNCmap(TFs_MSIG)[,2], HGNCmap(TFs_CISBP)[,2])))
use = intersect(allhg, activesym)
mymat = matrix(0, nr=length(use), nc=4)
rownames(mymat) = use
iu = function(x) intersect(x,use)
mymat[na.omit(iu(HGNCmap(TFs_GO)[,2])),1] = 1
mymat[na.omit(iu(HGNCmap(TFs_MSIG)[,2])),2] = 1
mymat[na.omit(iu(HGNCmap(TFs_HOCO)[,2])),3] = 1
mymat[na.omit(iu(HGNCmap(TFs_CISBP)[,2])),4] = 1
colnames(mymat) = c("GO", "MSigDb", "HOCO", "CISBP")
upset(data.frame(mymat),nsets=4,sets=c("MSigDb", "HOCO", "GO", "CISBP"), keep.order=TRUE, order.by="degree"
)
```

## Classification of transcription factors

As noted by @Weirauch2014, interpretation of the "function and evolution
of DNA sequences" is dependent on the analysis of sequence-specific
DNA binding domains.  These domains are dynamic and cell-type
specific (@Gertz2013).  Classifying TFs according to
features of the binding domain is an ongoing process of
increasing intricacy.  Figure \@ref(fig:TFclass)
shows excerpts of hierarchies of terms related to TF type
derived from GO (on the left) and 
[TFclass](http://tfclass.bioinf.med.uni-goettingen.de/) (@Wingender2018).
There is a disagreement between our enumeration of TFs based
on GO in Figure \@ref(fig:lkupset) and the 1919 shown in AmiGO,
as the latter includes a broader collection of receptor activities.

```{r TFclass, out.width='110%', fig.cap = 'Screenshots of AmiGO and TFClass hierarchy excerpts.',echo=FALSE}
knitr::include_graphics('AMIGOplus.png')
```

Table \@ref(tab:classtab) provides examples of frequently
encountered TF classifications in the CISBP and HOCOMOCO
catalogs.  The numerical components of the HOCOMOCO
classes correspond to TFClass subfamilies (@Wingender2018).

Table: (\#tab:classtab) Most frequently represented TF classes in CISBP and HOCOMOCO. Entries in columns Nc (Nh) are numbers of distinct TFs annotated to classes in columns CISBP (HOCOMOCO) respectively.  Entries are ordered top to bottom by frequency of occurrence.  There is no substantive correspondence between entries on a given row.  Harmonization of class terminology is beyond the scope of this paper.

```{r dodo,echo=FALSE}
library(knitr)
cismap = HGNCmap(TFs_CISBP)
scis = split(cismap, cismap$HGNC)
uf = vapply(scis, function(x) x$Family_Name[1],"character")
CISTOP = sort(table(uf),decreasing=TRUE)[1:10]
hoc = HGNCmap(TFs_HOCO)
shoc = split(hoc, hoc$HGNC)
sfam = vapply(shoc, function(x)x$`TF family`[1], "character")
HOTOP = sort(table(sfam),decreasing=TRUE)[1:10]
kable(data.frame(CISBP=names(CISTOP), Nc=as.numeric(CISTOP), 
   HOCOMOCO=names(HOTOP), Nh=as.numeric(HOTOP)), format="markdown")
```

## Enumerating TF targets 

The Broad Institute MSigDb (@Subramanian15545) includes
a gene set collection devoted to cataloging TF targets.
We have used Bioconductor's `r Biocpkg("GSEABase")` package
to import and serialize the `gmt` representation of this
collection.

```{r lkbro}
TFutils::tftColl
```

Names of TFs for which target sets are assembled are encoded 
in a systematic way, with underscores separating substrings
describing motifs, genes, and versions.
Some peculiarity in nomenclature in the MSigDb labels can
be observed:
```{r lktft2}
grep("NFK", names(TFutils::tftColl), value=TRUE)
```

Manual curation will be needed to improve the precision
with which MSigDb TF target sets can be associated with
specific TFs or motifs.

## Quantitative predictions of TF binding affinities

In this subsection we address representation of putative
binding sites.  First we illustrate how to represent sequence-based
affinity measures and the binding site locations implied by
these.  We then discuss use of results of ChIP-seq experiments
for cell-type-specific binding site enumeration.

**Affinity scores based on reference sequence.**
The FIMO algorithm of the MEME suite (@Grant2011) was used
to score the human reference genome for TF binding
affinity for `r nrow(fimoMap)` motif matrices
to which genes are associated.  Full
details are provided in @Sonawane2017.  Sixteen (16)
tabix-indexed BED files are lodged in an AWS S3
bucket for illustration purposes.

```{r demoredu}
library(GenomicFiles)
data(fimo16)
fimo16
head(colData(fimo16))
```

We harvest scores in a genomic interval of interest
(bound to `fimo16` in the `rowRanges` assignment
below) using `reduceByFile`.
This yields a list with one element per file.  Each such
element holds a list of `scanTabix` results, one per query range.
```{r demoredu2, cache=TRUE}
if (.Platform$OS.type != "windows") {
 library(BiocParallel)
 register(SerialParam()) # important for macosx?
 rowRanges(fimo16) = GRanges("chr17", IRanges(38.077e6, 38.084e6))
 rr = GenomicFiles::reduceByFile(fimo16, MAP=function(r,f)
   scanTabix(f, param=r))
} else {
rr = readRDS(system.file("fimoser/fimo16rr.rds", package="TFutils"))
}
```
scanTabix produces a list of vectors of text strings, which we parse
with `data.table::fread`.  The resulting tables are then
reduced to a genomic location and -log10 of the p-value derived
from the binding affinity statistic of FIMO in the vicinity
of that location.
```{r paraa}
asdf = function(x) data.table::fread(paste0(x, collapse="\n"), header=FALSE)
gg = lapply(rr, function(x) {
       tmp = asdf(x[[1]][[1]]) 
       data.frame(loc=tmp$V2, score=-log10(tmp$V7))
     })
for (i in 1:length(gg))  gg[[i]]$tf = colData(fimo16)[i,2]
```
It turns out there are too many distinct TFs
to display names individually, so we label the
scores with the names of the
associated TF families as defined in CISBP.
```{r domat}
matchcis = match(colData(fimo16)[,2], cisbpTFcat[,2])
famn = cisbpTFcat[matchcis,]$Family_Name
for (i in 1:length(gg))  gg[[i]]$tffam = famn[i]
nn = do.call(rbind, gg)
```

<!--
> odf = as.data.frame(reduce(genemodel("ORMDL3")))
'select()' returned 1:many mapping between keys and columns
> odf
  seqnames    start      end width strand
1    chr17 38077296 38078938  1643      -
2    chr17 38079365 38079516   152      -
3    chr17 38080283 38080478   196      -
4    chr17 38081008 38081058    51      -
5    chr17 38081422 38081624   203      -
6    chr17 38081876 38083094  1219      -
7    chr17 38083737 38083884   148      -
-->

A simple display of _predicted_ TF binding affinity near the
gene ORMDL3
is provided in Figure \@ref(fig:finish).

```{r finish, fig.height=3.5,echo=FALSE,fig.cap='TF binding in the vicinity of gene ORMDL3.  Points are -log10-transformed FIMO-based p-values colored according to TF class as annotated in CISBP.  Segments at bottom of plot are transcribed regions of ORMDL3 according to UCSC gene models in build hg19.'}
library(ggplot2)
myf = function(a=38077296, b=38078938) 
  geom_segment(aes(x=a, xend=b, y=2.85, yend=2.85, 
  colour="[ORMDL3]"))
ggplot(nn, aes(x=loc,y=score,group=tffam, colour=tffam)) + 
  geom_point() + myf() + myf(38079365, 38079516) + 
    myf(38080283, 38080478) + myf(38081008, 38081058) + 
    myf(38081422, 38081624) + myf(38081876, 38083094) + 
    myf(38083737, 38083884) + ylab("-log10 FIMO p-value") + 
 xlab("position on chr17, hg19")
```

**TF binding predictions based on ChIP-seq data from ENCODE.**
The ENCODE project provides BED-formatted
reports on ChIP-seq experiments for many
combinations of cell type and DNA-binding
factors.  TFutils includes a table `encode690`
that gives information on 690 experiments involving
pairs formed from
`r length(table(TFutils::encode690$cell))` cell lines and 
`r length(table(TFutils::encode690$target))` TFs for which
results have been recorded as GRanges
instances that can be
acquired with the `r Biocpkg("AnnotationHub")` package.
Positional relationships between cell-type specific binding
sites and genomic features can be investigated.  An
illustration is given in Figure \@ref(fig:lkbi), in which
is it suggested that in HepG2 cells, CEBPB exhibits a distinctive
pattern of binding in the vicinity of ORMDL3.  

```{r lkbi, fig.cap='Binding of CEBPB in the vicinity of ORMDL3 derived from ChIP-seq experiments in four cell lines reported by ENCODE.  Colored rectangles at top are regions identified as narrow binding peaks, arrows in bottom half are exons in ORMDL3.  Arrows sharing a common vertical position are members of the same transcript as cataloged in Ensembl version 75.', echo=FALSE}
pp = readPNG("ormdl3CEBPB.png")
grid.raster(pp)
```


## Summary

We have compared enumerations of human transcription factors
by different projects, provided access to two forms of
binding domain classification, and illustrated the use
of cloud-resident genome-wide binding predictions.
In the next section we review selected details of data structures
and methods of the `r Biocpkg("TFutils")` package.

# Methods 

## Implementation

The TFutils package is designed to lower barriers to
usage of key findings of TF biology in human genome
research.  TFutils is supplied as a conventional R package
distributed with, and making use of, the Bioconductor
software ecosystem.  TFutils includes ready-to-use
reference data, tools for visualizing binding sites,
and tools that simplify integrative use of TF binding
information with GWAS findings.

### Data resources

**Catalogs.** Two reference resources have been collected into
the TFutils package as data.frame instances.  These
are `cisbpTFcat` (CISBP: `r nrow(cisbpTFcat)` x `r ncol(cisbpTFcat)`),
and
`hocomoco.mono.sep2018` (mononucleotide models, full catalog,
`r nrow(hocomoco.mono.sep2018)` x `r ncol(hocomoco.mono.sep2018)`).  These
data.frames are snapshots of the CISBP and HOCOMOCO catalogs

**Indexed BED in AWS S3.**  As described above `fimo16`
provides programmatic access to FIMO scores for 16 TFs, using
the `r Biocpkg("GenomicFiles")` protocol.

**Annotated reference to ENCODE ChIP-seq results.**
`encode690` simplifies programmatic access to TF:cell-line
combinations available in Bioconductor `r Biocpkg("AnnotationHub")`.

**TF targets enumerated in MsigDb.**  The c3-TFT (TF targets) subset
from MSigDb is provided as a GeneSetCollection instance as
defined in `r Biocpkg("GSEABase")`.

**Illustrative GWAS records.** The full EBI/EMBL GWAS catalog
is available in the `r Biocpkg("gwascat")` package; for convenience,
an excerpt focusing on chromosome 17 is supplied with TFutils
as `gwascat_hg19_chr17`.

### Infrastructure for interacting with components of TFutils

**Interactive enumeration of TF targets implicated in GWAS.**
The `TFtargs` function runs a shiny app that permits
selection of a TF in the nomenclature of the MSigDb
c3/TFT gene set collection.
The app will search an object
provided by the `r Biocpkg("gwascat")` package for
references in the `MAPPED_GENE` field that match
the targets of the selected TF.
Figure \@ref(fig:lktarapp)
gives an illustration.  

```{r lktarapp, fig.cap='TFtargs() screenshot.  This example reports on recent EBI GWAS catalog hits on chromosome 17 only.',echo=FALSE}
pp = readPNG("tfTargsApp.png")
grid.raster(pp)
```

**The TFCatalog S4 class**.  Reference catalogs for TF biology are structured with the
`TFCatalog` S4 class.  Two essential components for
managing a catalog are the native TF identifier
for the catalog and the HGNC gene symbol typically
used to name the TF.  The `TFCatalog` class includes
a name field to name the catalog, and
a character vector with elements comprised of the
native identifiers for catalogued TFs.  
For example, CISBP uses `T004843_1.02` to refer
to motifs associated with gene TFAP2B.  There
are five such motifs, three derived from SELEX,
one from Transfac, and one from Hocomoco.

A `data.frame` instance that has an
obligatory column named 'HGNC' can include any collection
of fields that offer metadata about the TF in the specified
catalog.  Here is how we construct and view a TFCatalog object
using the CISBP reference data.
```{r ljlj}
data(cisbpTFcat)
TFs_CISBP = TFCatalog(name="CISBP.info", 
   nativeIds=cisbpTFcat[,1],
   HGNCmap = cisbpTFcat)
TFs_CISBP
```

## Operation: Use cases

In this section we consider applications of the tools
in genetic epidemiology.  First we look for TFs that
may harbor variants associated with traits in the EBI
GWAS catalog.  Then we show how to enumerate traits
associated with targets of a selected TF.

**TFs that are direct GWAS hits for a given trait.** `directHitsInCISBP` accepts
a string naming a trait , and returns a data.frame
of TFs identified as "mapped genes" for the trait,
with their TF "family name".

```{r defdir}
library(dplyr)
library(magrittr)
library(gwascat)
#data(ebicat37)
load(system.file("legacy/ebicat37.rda", package="gwascat"))
directHitsInCISBP("Rheumatoid arthritis", ebicat37)
```

**Traits mapped to genes that are targets of a given TF**

`topTraitsOfTargets` will acquire the targets of a selected
TF, check for hits in these genes in a given GWAS catalog instance, and
tabulate the most commonly reported traits.

```{r lktr}
tt = topTraitsOfTargets("MTF1", TFutils::tftColl, ebicat37)
head(tt)
table(tt[,1])
```

# Discussion

Sources and consequences of variations in DNA transcription
are fundamental problems for cell biology, and the projects we 
have made use of for cataloging
transcription factors are at the boundaries of current knowledge.

It is noteworthy that the four resources used for Figure
\@ref(fig:lkupset) agree on names of only 119 TFs.  The
fact that CISBP distinguishes 475 TFs that are not identified
in any other source should be better understood.
We observe that the ascription of TF status to AHRR is
based on its sharing motifs with AHR (see
\url{http://cisbp.ccbr.utoronto.ca/TFreport.php?searchTF=T014165_1.02}).

Figure \@ref(fig:TFclass) and Table \@ref(tab:classtab) 
show that the classification of TFs
is now fairly elaborate.  Use of the precise terminology of
the TFClass system to label TFs of interest
at present relies on associations provided
with the HOCOMOCO catalog.

As population studies in genomic and genetic epidemiology
grow in size and scope, principles for organizing and
prioritizing loci associated with phenotypes of interest
are urgently needed.  Figure \@ref(fig:lktarapp) shows
that loci associated with 
phenotypes related to kidney function, lung function,
and IL-8 levels are potentially unified through the fact
that the GWAS hits are connected with genes identified
as targets of VDR (vitamin D receptor).  This example
limited attention to hits on chromosome 17; the `TFtargs` tool
permits \textit{ad libitum} exploration of 
phenotype-locus-gene-TF associations.
Our hope is that
the tools and resources collected in TFutils will foster
systematic development of evidence-based mechanistic
network models for transcription regulation in human
disease contexts, thereby
contributing to the development of personalized genomic medicine.

# Acknowledgments

Support for the development of this software was provided by NIH grants 
U01 CA214846 (Carey, PI), U24 CA180996 (Morgan, PI),
Chan Zuckerberg Initiative DAF 2018-183436 (Carey, PI), and
R01 NHLBI HL118455 (Raby, PI).

# Session Information

```{r dose}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{metadata_tf}
\alias{metadata_tf}
\title{metadata_tf: list with metadata (motif_if and hgnc_symbol) about all the CISBP FIMO scan TF bed files}
\format{
list
}
\source{
K. Glass ran FIMO
}
\usage{
metadata_tf
}
\description{
metadata_tf: list with metadata (motif_if and hgnc_symbol) about all the CISBP FIMO scan TF bed files
}
\examples{
TFutils::metadata_tf
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tftCollMap}
\alias{tftCollMap}
\title{tftCollMap: data.frame with information on MSigDb TFs for human}
\format{
data.frame
}
\source{
\url{http://software.broadinstitute.org/gsea/msigdb/genesets.jsp?collection=TFT}
}
\usage{
tftCollMap
}
\description{
tftCollMap: data.frame with information on MSigDb TFs for human
}
\note{
Annotation of TFs is ad-hoc.  GeneSet names were tokenized, splitting
by underscore, and then fragments were matched to SYMBOL and ALIAS elements
of org.Hs.eg.db.  Extracted March 2018
}
\examples{
head(TFutils::tftCollMap)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lamget.R
\name{browse_lambert_main}
\alias{browse_lambert_main}
\title{use DT::datatable to browse the Lambert table S1}
\usage{
browse_lambert_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
result of DT::datatable
}
\description{
use DT::datatable to browse the Lambert table S1
}
\note{
PMIDs are converted to HTML anchors and DT::datatable is run with \code{escape=FALSE}.
}
\examples{
if (interactive()) browse_lambert_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{encode690}
\alias{encode690}
\title{encode690: DataFrame extending AnnotationHub metadata about ENCODE cell line x TF ranges}
\format{
DataFrame
}
\source{
see metadata(encode690)
}
\usage{
encode690
}
\description{
encode690: DataFrame extending AnnotationHub metadata about ENCODE cell line x TF ranges
}
\examples{
names(TFutils::encode690)
TFutils::encode690[,1:5]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fimo16}
\alias{fimo16}
\title{fimo16: GenomicFiles instance to AWS S3-resident FIMO bed for 16 TFs}
\format{
GenomicFiles for a TabixFileList
}
\source{
K. Glass FIMO runs, see \url{https://doi.org/10.1016/j.celrep.2017.10.001}
}
\usage{
fimo16
}
\description{
fimo16: GenomicFiles instance to AWS S3-resident FIMO bed for 16 TFs
}
\examples{
TFutils::fimo16
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambert_ccbr_get.R
\name{browse_humantfs_main}
\alias{browse_humantfs_main}
\title{use DT::datatable to browse the Lambert's Human Transcription Factors repository}
\usage{
browse_humantfs_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
result of DT::datatable
}
\description{
use DT::datatable to browse the Lambert's Human Transcription Factors repository
}
\examples{
if (interactive()) browse_humantfs_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfcirc.R
\name{defaultCircosParms}
\alias{defaultCircosParms}
\title{basic layout parameters for circos}
\usage{
defaultCircosParms()
}
\value{
a list
}
\description{
basic layout parameters for circos
}
\examples{
head(defaultCircosParms())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfutils.R
\name{importFIMO_local_split}
\alias{importFIMO_local_split}
\title{utility to read FIMO outputs from local resource(cluster), assuming bed text split by chromosome}
\usage{
importFIMO_local_split(tf, chr)
}
\arguments{
\item{tf}{character(1) file id}

\item{chr}{character(1) chromosome name}
}
\value{
data.table instance
}
\description{
utility to read FIMO outputs from local resource(cluster), assuming bed text split by chromosome
}
\examples{
requireNamespace("GenomicRanges")
requireNamespace("IRanges")
importFIMO_local_split("M5946_1", "chr1")
dim(importFIMO_local_split("M5946_1", "chr17"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{demo_fimo_granges}
\alias{demo_fimo_granges}
\title{a list of GRanges instances with TF FIMO scores returned by \code{fimo_granges}}
\format{
a list of GRanges instances
}
\usage{
demo_fimo_granges
}
\description{
a list of GRanges instances with TF FIMO scores returned by \code{fimo_granges}
}
\examples{
names(S4Vectors::mcols(demo_fimo_granges$VDR[[1]]))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tfhash}
\alias{tfhash}
\title{tfhash: data.frame with MSigDb TFs, TF targets as symbol or ENTREZ}
\format{
list
}
\source{
MSigDb "c3" (motif gene sets) has been harvested for simple annotation of TFs and targets.
}
\usage{
tfhash
}
\description{
tfhash: data.frame with MSigDb TFs, TF targets as symbol or ENTREZ
}
\examples{
TFutils::tfhash
tfhash[1:3,]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tasks.R
\name{URL_s3_tf}
\alias{URL_s3_tf}
\title{utility to generate link to biocfound bucket for FIMO TFBS scores}
\usage{
URL_s3_tf(tag = "M3433")
}
\arguments{
\item{tag}{character(1) token identifying TF, can be an HGNC gene name or Mnnnn PWM tag.
It must be findable in TFutils::fimoMap table.}
}
\value{
character(1) URL
}
\description{
utility to generate link to biocfound bucket for FIMO TFBS scores
}
\examples{
URL_s3_tf
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/importFIMO.R
\name{importFIMO,TabixFile,GRanges-method}
\alias{importFIMO,TabixFile,GRanges-method}
\alias{importFIMO}
\alias{importFIMO,character,missing-method}
\title{import a FIMO bed-like file}
\usage{
\S4method{importFIMO}{TabixFile,GRanges}(src, parms, ...)

\S4method{importFIMO}{character,missing}(src, parms, ...)
}
\arguments{
\item{src}{TabixFile instance}

\item{parms}{a GRanges instance delimiting the import; multiple GRanges can be used}

\item{\dots}{passed to GenomicRanges::GRanges}
}
\value{
instance of GRanges
}
\description{
import a FIMO bed-like file
}
\examples{
if (requireNamespace("Rsamtools")) {
 tf = Rsamtools::TabixFile(system.file("M5946_1/chr1.bed.gz", package="TFutils"))
 importFIMO(tf, GenomicRanges::GRanges("chr1", IRanges::IRanges(1e6,11e6)))
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{named_tf}
\alias{named_tf}
\title{named_tf: named list with the names being the hgnc_symbol of the motif_id}
\format{
list
}
\source{
K. Glass ran FIMO
}
\usage{
named_tf
}
\description{
named_tf: named list with the names being the hgnc_symbol of the motif_id
}
\examples{
TFutils::named_tf
named_tf[["VDR"]]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genepi.R
\name{directHitsInCISBP}
\alias{directHitsInCISBP}
\title{demonstrate interoperation of TF catalog with GWAS catalog}
\usage{
directHitsInCISBP(traitTag, gwascat)
}
\arguments{
\item{traitTag}{character(1) string found in DISEASE/TRAIT field of gwascat instance}

\item{gwascat}{instance of \code{\link[gwascat]{gwaswloc-class}}}
}
\value{
data.frame
}
\description{
demonstrate interoperation of TF catalog with GWAS catalog
}
\examples{
data(gwascat_hg19_chr17)
directHitsInCISBP("Prostate cancer" , gwascat_hg19_chr17)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gob.R
\name{TFtargs}
\alias{TFtargs}
\title{gadget to help sort through tags naming TFs}
\usage{
TFtargs(
  gscoll = TFutils::tftColl,
  initTF = "VDR_Q3",
  gwcat = TFutils::gwascat_hg19_chr17,
  gadtitle =
    "Search for a TF; its targets will be checked for mapped status in GWAS catalog"
)
}
\arguments{
\item{gscoll}{a GSEABase GeneSetCollection}

\item{initTF}{character(1) initial TF string for app}

\item{gwcat}{GRanges-like structure with GWAS catalog information}

\item{gadtitle}{character(1) a title for the gadget panel}
}
\value{
on app conclusion a data.frame is returned
}
\description{
gadget to help sort through tags naming TFs
}
\note{
Will use TFutils::gwascat_hg19_chr17 to look for 'MAPPED_GENE' field entries matching targets, also hardcoded to use org.Hs.eg.db to map symbols
}
\examples{
if (interactive()) TFtargs()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/go_tf_get.R
\name{gotf_url}
\alias{gotf_url}
\title{Update Oct 18 2021}
\usage{
gotf_url()
}
\description{
Update Oct 18 2021
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lambert_snps}
\alias{lambert_snps}
\title{lambert_snps is Table S3 of Lambert et al PMID 29425488}
\format{
data.frame
}
\usage{
lambert_snps
}
\description{
lambert_snps is Table S3 of Lambert et al PMID 29425488
}
\examples{
head(lambert_snps)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cisbpTFcat}
\alias{cisbpTFcat}
\title{cisbpTFcat: data.frame with information on CISBP TFs for human, retained for reproducibility support; see
cisbpTFcat_2.0 for a more recent catalog}
\format{
data.frame
}
\source{
\url{http://cisbp.ccbr.utoronto.ca/bulk.php} select Homo_sapiens
}
\usage{
cisbpTFcat
}
\description{
cisbpTFcat: data.frame with information on CISBP TFs for human, retained for reproducibility support; see
cisbpTFcat_2.0 for a more recent catalog
}
\note{
Extracted March 2018, checked August 2018.  The only changes
observed are that genes ZUFSP and T are used has HGNC values
in the March catalog; these symbols seem to be absent from the
org.Hs.eg.db of August 2018.  The records involved are
1356, 7412 and 7413.  These symbols were left in the package image
of CISBP in August 2018.
}
\examples{
head(TFutils::cisbpTFcat)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse_lambert_gwaslinks.R
\name{browse_lambert_gwaslinks}
\alias{browse_lambert_gwaslinks}
\title{browse several hundred disease-TF associations with hyperlinked PMIDs}
\usage{
browse_lambert_gwaslinks()
}
\value{
DT::datatable
}
\description{
browse several hundred disease-TF associations with hyperlinked PMIDs
}
\note{
Based on supplemental table S4 of PMID 29425488
}
\examples{
if (interactive()) browse_lambert_gwaslinks()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfcirc.R
\name{tffamCirc.plot}
\alias{tffamCirc.plot}
\title{use a radial plot (by default) for motif stack}
\usage{
tffamCirc.plot(motiflist, circosParms = defaultCircosParms())
}
\arguments{
\item{motiflist}{a list of pfm instances from motifStack}

\item{circosParms}{a list of parameter settings for circos plot}
}
\value{
side effect to graphics device
}
\description{
use a radial plot (by default) for motif stack
}
\examples{
p1 = tffamCirc.prep( )
tffamCirc.plot(p1[c(1:8, 10:17, 19)])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tfcirc.R
\name{tffamCirc.prep}
\alias{tffamCirc.prep}
\title{set up list of pfms in motifStack protocol}
\usage{
tffamCirc.prep(tffam = "Paired-related HD factors{3.1.3}", trimfac = 0.4)
}
\arguments{
\item{tffam}{character(1) name of TF family as found in TFutils::hocomoco.mono field \verb{TF family}}

\item{trimfac}{fraction passed as parameter \code{t} to motifStack::trimMotif}
}
\value{
a list of pfm instances as defined in motifStack
}
\description{
set up list of pfms in motifStack protocol
}
\note{
Uses MotifDb, motifStack to create a list of pfms
}
\examples{
n1 = tffamCirc.prep()
str(n1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/go_tf_get.R
\name{retrieve_gotf_main}
\alias{retrieve_gotf_main}
\title{acquire the content of Table S1.A from Lovering et al., A GO catalogue of human DNA-binding
transcription factors, DOI: https://doi.org/10.1101/2020.10.28.359232}
\usage{
retrieve_gotf_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
a tbl_df
}
\description{
acquire the content of Table S1.A from Lovering et al., A GO catalogue of human DNA-binding
transcription factors, DOI: https://doi.org/10.1101/2020.10.28.359232
}
\note{
This will download the spreadsheet if not found in \code{cache}.
}
\examples{
if (interactive()) retrieve_gotf_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TFCatalog-class.R
\name{show,TFCatalog-method}
\alias{show,TFCatalog-method}
\title{produce a concise report on TFCatalog instance}
\usage{
\S4method{show}{TFCatalog}(object)
}
\arguments{
\item{object}{instance of TFCatalog}
}
\value{
side effect
}
\description{
produce a concise report on TFCatalog instance
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambert_ccbr_get.R
\name{retrieve_humantfs_main}
\alias{retrieve_humantfs_main}
\title{acquire the CSV content for table S1 of Lambert et al. Cell 2018 from the Human TFS repository at http://humantfs.ccbr.utoronto.ca}
\usage{
retrieve_humantfs_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
a tbl_df
}
\description{
acquire the CSV content for table S1 of Lambert et al. Cell 2018 from the Human TFS repository at http://humantfs.ccbr.utoronto.ca
}
\note{
This will download the spreadsheet if not found in \code{cache}.
}
\examples{
if (interactive()) retrieve_humantfs_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lamget.R
\name{retrieve_lambert_main}
\alias{retrieve_lambert_main}
\title{acquire the Excel spreadsheet content for table S1 of Lambert et al. Cell 2018, "The Human Transcription Factors"}
\usage{
retrieve_lambert_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
a tbl_df
}
\description{
acquire the Excel spreadsheet content for table S1 of Lambert et al. Cell 2018, "The Human Transcription Factors"
}
\note{
This will download the spreadsheet if not found in \code{cache}.
}
\examples{
if (interactive()) retrieve_lambert_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lambert_ccbr_get.R
\name{ccbr_cell_url}
\alias{ccbr_cell_url}
\title{Update Oct 18 2021}
\usage{
ccbr_cell_url()
}
\description{
Update Oct 18 2021
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tftColl}
\alias{tftColl}
\title{tftColl: GSEABase GeneSetCollection for transcription factor targets}
\format{
GSEABase GeneSetCollection instance
}
\source{
broad institute
}
\usage{
tftColl
}
\description{
tftColl: GSEABase GeneSetCollection for transcription factor targets
}
\note{
run GSEABase::getGMT() on c3/TFT geneset collection
from MSigDb
}
\examples{
TFutils::tftColl
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup_lambert.R
\name{anchor_pmids}
\alias{anchor_pmids}
\title{check columns of a dataframe for numerical tokens of 7 or 8 digits
and create HTML anchors to pubmed.gov constituting a link to a PMID}
\usage{
anchor_pmids(dataframe)
}
\arguments{
\item{dataframe}{a data.frame instance}
}
\value{
data.frame with HTML anchors to pubmed.gov inserted where 7- or 8-digit numbers are found
}
\description{
check columns of a dataframe for numerical tokens of 7 or 8 digits
and create HTML anchors to pubmed.gov constituting a link to a PMID
}
\note{
The method of isolating putative PMIDs is peculiar to patterns found in
the comment fields of annotated TF table (supplemental table S1 found in
\url{https://www.cell.com/cms/10.1016/j.cell.2018.01.029/attachment/88c0eca1-66f9-4068-b02e-bd3d55144f79/mmc2.xlsx} of PMID 29425488).  When DT::datatable is called on the output
of this function with \code{escape=FALSE} the PMIDs will render as hyperlinks.
Note that column 1 is assumed to be an ENSEMBL ID which could have 7 or 8 digits but is handled differently
}
\examples{
litdf = data.frame(id="ENSG00000116819", a="Binds the same GCCTGAGGC sequence as the other AP-2s (PMID: 24789576)",
     stringsAsFactors=FALSE)
anchor_pmids(litdf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ensRsLoc.R
\name{get_rslocs_38}
\alias{get_rslocs_38}
\title{utility to obtain location etc. for rsids of SNPs}
\usage{
get_rslocs_38(rsids = c("rs6060535", "rs56116432"))
}
\arguments{
\item{rsids}{character vector of dbSNP identifiers}
}
\value{
GRanges instance
}
\description{
utility to obtain location etc. for rsids of SNPs
}
\note{
Uses rest.ensembl.org, posting to variant_recorder/homo_sapiens.
Parses result minimally, using only the first SPDI to obtain
location information, adding 1 as ensembl genomic coordinates
are zero-based.
}
\examples{
if (interactive()) get_rslocs_38() # see https://stat.ethz.ch/pipermail/bioc-devel/2020-October/017263.html
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TFCatalog-class.R
\name{TFCatalog}
\alias{TFCatalog}
\title{Constructor for TFCatalog}
\usage{
TFCatalog(name, nativeIds, HGNCmap, metadata)
}
\arguments{
\item{name}{informative character(1) for collection}

\item{nativeIds}{character() vector of identifiers used by collection creators}

\item{HGNCmap}{data.frame with column 1 nativeIds, column 2 HGNC or hgnc.heur for MSigDb
and any other columns of use}

\item{metadata}{a list of metadata elements}
}
\value{
instance of TFCatalog
}
\description{
Constructor for TFCatalog
}
\examples{
if (require("GSEABase")) {
 TFs_MSIG = TFCatalog(name="MsigDb.TFT",nativeIds=names(TFutils::tftColl),
 HGNCmap=data.frame(TFutils::tftCollMap,stringAsFactors=FALSE))
 TFs_MSIG
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TFCatalog-class.R
\name{HGNCmap}
\alias{HGNCmap}
\title{simple accessor for HGNCmap component of TFCatalog}
\usage{
HGNCmap(x)
}
\arguments{
\item{x}{instance of TFCatalog}
}
\value{
dataframe instance
}
\description{
simple accessor for HGNCmap component of TFCatalog
}
\examples{
HGNCmap
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genemodel2.R
\name{genemodelDF}
\alias{genemodelDF}
\title{use EnsDb to generate an exon-level model of genes identified by symbol}
\usage{
genemodelDF(sym, resource, columnsKept = c("gene_id", "tx_id"), ...)
}
\arguments{
\item{sym}{a character() vector of gene symbols}

\item{resource}{should be or inherit from EnsDb,
answering exons(), with AnnotationFilter::SymbolFilter
as filter parameter}

\item{columnsKept}{character vector used as \code{columns} param in exons()}

\item{\dots}{passed to exons()}
}
\value{
data.frame instance with exons in rows
}
\description{
use EnsDb to generate an exon-level model of genes identified by symbol
}
\note{
There are many approaches available to acquiring 'gene models'
in Bioconductor; this one emphasizes the use of the exons method
for Ensembl annotation.
}
\examples{
if (requireNamespace("EnsDb.Hsapiens.v75")) {
 orm = genemodelDF("ORMDL3", EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75)
 dim(orm)
}
head(orm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fimo_granges.R
\name{fimo_granges}
\alias{fimo_granges}
\title{create a list of GRanges for FIMO hits in a GenomicFiles instance, corresponding to a GRanges-based query}
\usage{
fimo_granges(gf, query)
}
\arguments{
\item{gf}{GenomicFiles instance, like fimo16 in TFutils}

\item{query}{a GRanges specifying ranges to check for TF binding scores}
}
\value{
a list of GRanges, produced by GenomicFiles::reduceByRange
}
\description{
create a list of GRanges for FIMO hits in a GenomicFiles instance, corresponding to a GRanges-based query
}
\note{
Be sure to use \verb{register([BPPARAM])} appropriately.
}
\examples{
if (interactive()) {   # need internet
 # setup -- annotate fimo16 object and create an informative
 # query
 colnames(fimo16) = fimo16$HGNC
 si = Seqinfo::Seqinfo(genome="hg19")["chr17"] # to fix query genome
 myg = GRanges("chr17", IRanges(38.07e6,38.09e6), seqinfo=si)
 requireNamespace("BiocParallel")
 BiocParallel::register(BiocParallel::SerialParam())
 f1 = fimo_granges(fimo16[, c("VDR", "POU2F1")], myg)
 f1
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fimoMap}
\alias{fimoMap}
\title{fimoMap: table with Mnnnn (motif PWM tags) and HGNC symbols for TFs}
\format{
data.frame
}
\source{
Kimberly Glass (rekrg@channing.harvard.edu)
}
\usage{
fimoMap
}
\description{
fimoMap: table with Mnnnn (motif PWM tags) and HGNC symbols for TFs
}
\examples{
head(TFutils::fimoMap)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{seqinfo_hg19_chr17}
\alias{seqinfo_hg19_chr17}
\title{a Seqinfo instance for a chr17 in hg19}
\format{
a Seqinfo instance
}
\usage{
seqinfo_hg19_chr17
}
\description{
a Seqinfo instance for a chr17 in hg19
}
\examples{
seqinfo_hg19_chr17
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gwascat_hg19_chr17}
\alias{gwascat_hg19_chr17}
\title{gwascat_hg19: GRanges of march 21 2018 EBI gwascat, limit to chr17}
\format{
GenomicRanges GRanges instance
}
\source{
gwascat::makeCurrentGwascat, with gwascat:::lo38to19 applied
}
\usage{
gwascat_hg19_chr17
}
\description{
gwascat_hg19: GRanges of march 21 2018 EBI gwascat, limit to chr17
}
\examples{
TFutils::gwascat_hg19_chr17[,1:5]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genemodel2.R
\name{genemodForGviz}
\alias{genemodForGviz}
\title{create a GeneRegionTrack instance for selected symbols}
\usage{
genemodForGviz(
  sym = "ORMDL3",
  id_elem = c("symbol", "tx_id"),
  resource = EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75,
  ...
)
}
\arguments{
\item{sym}{character vector of gene symbols, should be neighboring genes}

\item{id_elem}{vector of names of columns generated by genemodelDF to be
used to label transcripts}

\item{resource}{should be or inherit from EnsDb,
answering exons(), with AnnotationFilter::SymbolFilter
as filter parameter}

\item{\dots}{passed to genemodelDF}
}
\value{
instance of Gviz GeneRegionTrack
}
\description{
create a GeneRegionTrack instance for selected symbols
}
\note{
This function helps to display the locations of TF binding sites
in the context of complex gene models.  A complication is that we
have nice visualization of quantitative affinity predictions for TFs
in the vignette, based on ggplot2, but it is not clear how to use
that specific code to work with Gviz.
}
\examples{
if (requireNamespace("EnsDb.Hsapiens.v75") &
    requireNamespace("Gviz")) {
 orm = genemodForGviz("ORMDL3", resource= EnsDb.Hsapiens.v75::EnsDb.Hsapiens.v75)
 orm
 Gviz::plotTracks(orm, showId=TRUE) # change id_elem for shorter id string
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hocomoco.mono.sep2018}
\alias{hocomoco.mono.sep2018}
\title{hocomoco.mono.sep2018: data.frame with information on HOCOMOCO TFs for human, Sept 2018 download}
\format{
data.frame
}
\source{
\url{http://hocomoco11.autosome.ru/human/mono?full=true}
}
\usage{
hocomoco.mono.sep2018
}
\description{
hocomoco.mono.sep2018: data.frame with information on HOCOMOCO TFs for human, Sept 2018 download
}
\note{
Extracted September 2018
}
\examples{
head(TFutils::hocomoco.mono.sep2018)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/go_tf_get.R
\name{browse_gotf_main}
\alias{browse_gotf_main}
\title{use DT::datatable to browse the GO catalogue of human DNA-binding transcription factors in Table S1.A of Lovering et al.}
\usage{
browse_gotf_main(cache = BiocFileCache::BiocFileCache(ask = FALSE))
}
\arguments{
\item{cache}{a BiocFileCache instance}
}
\value{
result of DT::datatable
}
\description{
use DT::datatable to browse the GO catalogue of human DNA-binding transcription factors in Table S1.A of Lovering et al.
}
\examples{
if (interactive()) browse_gotf_main()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grabTab.R
\name{grabTab}
\alias{grabTab}
\title{create table of TF targets and related metadata}
\usage{
grabTab(
  tfstub = "STAT1",
  gscoll = TFutils::tftColl,
  orgdb = org.Hs.eg.db::org.Hs.eg.db,
  gwrngs = TFutils::gwascat_hg19_chr17
)
}
\arguments{
\item{tfstub}{character(1) gene-like symbol for TF; will be grepped in names(gscoll)}

\item{gscoll}{a GSEABase GeneSetCollection}

\item{orgdb}{an instance of OrgDb as defined in AnnotationDbi}

\item{gwrngs}{a GRanges representing EBI gwascat, must have \code{DISEASE/TRAIT}, \code{MAPPED_GENE}}
}
\value{
data.frame instance
}
\description{
create table of TF targets and related metadata
}
\note{
This function will link together information on targets of a
given TF to the GWAS catalog.
}
\examples{
gt = grabTab("VDR", gscoll=TFutils::tftColl,
   orgdb=org.Hs.eg.db::org.Hs.eg.db, gwrngs=TFutils::gwascat_hg19_chr17)
dim(gt)
head(gt)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cisbpTFcat_2.0}
\alias{cisbpTFcat_2.0}
\title{cisbpTFcat_2.0: data.frame with information on CISBP TFs for human, described in PMID 31133749}
\format{
data.frame
}
\source{
\url{http://cisbp.ccbr.utoronto.ca/bulk.php} select Homo_sapiens
}
\usage{
cisbpTFcat_2.0
}
\description{
cisbpTFcat_2.0: data.frame with information on CISBP TFs for human, described in PMID 31133749
}
\note{
Extracted August 2019.
}
\examples{
head(TFutils::cisbpTFcat_2.0)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hocomoco.mono}
\alias{hocomoco.mono}
\title{hocomoco.mono: data.frame with information on HOCOMOCO TFs for human}
\format{
data.frame
}
\source{
\url{http://hocomoco11.autosome.ru/human/mono?full=true}
}
\usage{
hocomoco.mono
}
\description{
hocomoco.mono: data.frame with information on HOCOMOCO TFs for human
}
\note{
Extracted March 2018
}
\examples{
head(TFutils::hocomoco.mono)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genepi.R
\name{topTraitsOfTargets}
\alias{topTraitsOfTargets}
\title{Use MSigDB TF targets resource to find targets of input TF and find traits to which these targets have been mapped}
\usage{
topTraitsOfTargets(TFsym, gsc, gwcat, ntraits = 6, force = FALSE, ...)
}
\arguments{
\item{TFsym}{character(1) symbol for a TF must be present in \code{tftCollMap[, "hgnc.heur"]}}

\item{gsc}{an instance of \code{\link[GSEABase]{GeneSetCollection-class}}, intended to enumerate
targets of a single transcription factor in each GeneSet, as in TFutils::tftColl}

\item{gwcat}{instance of \code{\link[gwascat]{gwaswloc-class}}}

\item{ntraits}{numeric(1) number of traits to report}

\item{force}{logical see note, set to true if you want to skip mapping from TFsym to a specific motif or TF identifier used as name of a GeneSet in gsc}

\item{\dots}{character() vector of fields in mcols(gwcat) to include}
}
\value{
data.frame
symbol, set force = TRUE to use a known 'motif' name among \code{names(gsc)}
}
\description{
Use MSigDB TF targets resource to find targets of input TF and find traits to which these targets have been mapped
}
\note{
If \code{tftCollMap[, "hgnc.heur"]} does not possess the necessary
}
\examples{
suppressPackageStartupMessages({
library(GSEABase)
})  # more results if you substitute ebicat37 from gwascat below
topTraitsOfTargets("MTF1" , tftColl, gwascat_hg19_chr17)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TFCatalog-class.R
\docType{class}
\name{TFCatalog-class}
\alias{TFCatalog-class}
\alias{"TFCatalog-class"}
\title{define a structure to hold information about TFs from diverse reference sources}
\description{
define a structure to hold information about TFs from diverse reference sources
}
\section{Slots}{

\describe{
\item{\code{name}}{character}

\item{\code{nativeIds}}{character tokens used by the provider to enumerate transcription factors}

\item{\code{HGNCmap}}{data.frame with atleast two columns,
native id as first column and HGNC symbol as second column}

\item{\code{metadata}}{ANY}
}}

\note{
This class respects the notions that 1) a source of information
about transcription factors should have a name, 2) each source
has its own 'native' nomenclature for the factors themselves,
3) it is common to use the gene symbol to refer to the transctiption
factor, and 4) additional metadata will frequently be required to
establish information about provenance of assertions about transcription
factors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setupHIZE.R
\name{setupHIZE}
\alias{setupHIZE}
\title{process a gene_attribute_matrix.txt file from harmonizeome into a GeneSetCollection}
\usage{
setupHIZE(txtfn = "gene_attribute_matrix.txt", tag)
}
\arguments{
\item{txtfn}{character(1) path to gene_attribute_matrix.txt file from harmonizeome}

\item{tag}{character(1) will be added to shortDescription field of each GeneSet instance}
}
\value{
GSEABase::GeneSetCollection
}
\description{
process a gene_attribute_matrix.txt file from harmonizeome into a GeneSetCollection
}
\note{
After uncompressing content of \url{http://amp.pharm.mssm.edu/static/hdfs/harmonizome/data/cheappi/gene_attribute_matrix.txt.gz} run this on gene_attribute_matrix.txt with tag="CHEA".
}
